-- AVIRAW - An AVI reader/writer library for Euphoria 2.1+ 
-- Platform independent version. 
-- /Mic, 2002
--
-- Update 20021008:
-- 	* Fixed AVI_Load and AVI_GetNextFrame so that they will handle uncompressed AVIs
-- 	  created by Virtualdub.
--
--	* Fixed AVI_Load so that it handles the case where there is no suggested size for the
--	  input buffer.
--
--	* Added support for another codec (Lossy CrunchYUV).
--	
--	* Added AVI_GetLastErrorMessage()
--
-- Update 20021001:
-- 	* Added some routines for reading AVIs
--
-- Update 20020930:
-- 	* Added support for compression (only 1 codec available atm)
--


include file.e
include get.e
include machine.e

-- Codecs
include crunch.e
include lcrunch.e


global constant AVI_NO_COMPRESSION = 0,
		AVI_CRUNCH = 1,
		AVI_LCRUNCH = 2,
		AVI_UNKNOWN = -1

constant AVI_compressorNames =
{
	"No compression",
	"CrunchYUV",
	"Lossy CrunchYUV"
}

global constant AVI_FRAMEFORMAT_WIDTH = 1,
		AVI_FRAMEFORMAT_HEIGHT = 2,
		AVI_FRAMEFORMAT_BITCOUNT = 3
		
		
sequence AVI_filename,
	 AVI_frameformat,
	 AVI_frameformatIn,
	 AVI_framesizes,
	 AVI_lasterrormessage

integer AVI_framerate,		-- for output
	AVI_framerateIn,	-- for input
	AVI_fn,			-- for output
	AVI_fnIn,		-- for input
	AVI_framesize,
	AVI_largestframe,
	AVI_totframesize,
	AVI_numframes,
	AVI_numframesIn,
	AVI_filesizeIn,
	AVI_inputpos,
	AVI_compressor,
	AVI_decompressor,
	AVI_suggbuffersize,	-- suggested buffer size for input files
	AVI_streams,		-- # of streams in input files
	AVI_indexpos,		-- position of index in input files
	AVI_movipos

atom AVI_compressiontime,
     AVI_inputbuffer
atom compressed



-- Convert a sequence of bytes to a 16-bit integer, and vice versa
--
function AVI_WORD(object w)
	if integer(w) then
		return {and_bits(w,#FF),floor(and_bits(w,#FF00)/#100)}
	end if
	return w[1]+(w[2]*256)
end function


-- Convert a sequence of bytes to a 32-bit integer, and vice versa
--
function AVI_DWORD(object dw)
	if sequence(dw) then
		return dw[1]+(dw[2]*#100)+(dw[3]*#10000)+(dw[4]*#1000000)
	end if
	return {and_bits(dw,#FF),floor(and_bits(dw,#FF00)/#100),
	        floor(and_bits(dw,#FF0000)/#10000),floor(and_bits(dw,#FF000000)/#1000000)}
end function



-- Set the name of the AVI file to write
--
global procedure AVI_SetFileName(sequence filename)
	AVI_filename = filename
end procedure



-- Set the output frame rate
--
global procedure AVI_SetFrameRate(integer rate)
	AVI_framerate = rate
end procedure



-- Set the output frame format ({width, height, bitsperpixel})
--
global procedure AVI_SetFrameFormat(integer fWidth,integer fHeight,integer fBpp)
	AVI_frameformat = {fWidth,fHeight,fBpp}
end procedure



-- Set the output compressor
--
global procedure AVI_SetCompressor(integer comp)
	if comp=AVI_CRUNCH then
		AVI_compressor = AVI_CRUNCH
	elsif comp=AVI_LCRUNCH then
		AVI_compressor = AVI_LCRUNCH
	else
		AVI_compressor = AVI_NO_COMPRESSION
	end if
end procedure



-- Initialize AVI output
--
global function AVI_Begin()
	integer padding
	
	if and_bits(AVI_frameformat[1],3) or
	   and_bits(AVI_frameformat[2],1) or
	   AVI_frameformat[3]!=24 then
	   	AVI_lasterrormessage = "Bad format. Make sure the width is divisible by 4"
	   	return 0
	end if
	
	AVI_fn = open(AVI_filename,"wb")
	if AVI_fn<0 then
		AVI_lasterrormessage = "Unable to open file for output: "&AVI_filename
		return 0
	end if

	AVI_numframes = 0
	AVI_framesize = AVI_frameformat[1]*AVI_frameformat[2]*floor((AVI_frameformat[3]+1)/8)
	
	-- Ugly code for writing various headers. Seems to work just fine..*cough*..
	
	puts(AVI_fn,"RIFF"&{0,0,0,0}&"AVI LIST"&{#C0,0,0,0}&"hdrlavih"&{#38,0,0,0})
	puts(AVI_fn,AVI_DWORD(floor(1000000/AVI_framerate)))
	puts(AVI_fn,AVI_DWORD(AVI_framesize))
	puts(AVI_fn,AVI_DWORD(0)&{#10,8,0,0})
	puts(AVI_fn,{0,0,0,0,0,0,0,0,1,0,0,0})
	puts(AVI_fn,AVI_DWORD(AVI_framesize))
	puts(AVI_fn,AVI_DWORD(AVI_frameformat[1]))
	puts(AVI_fn,AVI_DWORD(AVI_frameformat[2]))
	for i=1 to 4 do
		puts(AVI_fn,AVI_DWORD(0))
	end for
	puts(AVI_fn,"LIST"&{#74,0,0,0}&"strlstrh"&{#38,0,0,0}&"vids")
	for i=1 to 4 do
		puts(AVI_fn,AVI_DWORD(0))
	end for
	puts(AVI_fn,AVI_DWORD(1))
	puts(AVI_fn,AVI_DWORD(AVI_framerate))
	puts(AVI_fn,AVI_DWORD(0))
	puts(AVI_fn,AVI_DWORD(0))
	puts(AVI_fn,AVI_DWORD(AVI_framesize))
	puts(AVI_fn,AVI_DWORD(0))
	puts(AVI_fn,AVI_DWORD(0))
	puts(AVI_fn,AVI_WORD(0))
	puts(AVI_fn,AVI_WORD(0))
	puts(AVI_fn,AVI_WORD(AVI_frameformat[1]))
	puts(AVI_fn,AVI_WORD(AVI_frameformat[2]))
	puts(AVI_fn,"strf"&{#28,0,0,0,#28,0,0,0})
	puts(AVI_fn,AVI_DWORD(AVI_frameformat[1]))	-- biWidth
	puts(AVI_fn,AVI_DWORD(AVI_frameformat[2]))	-- biHeight
	puts(AVI_fn,AVI_WORD(1))			-- biPlanes
	puts(AVI_fn,AVI_WORD(AVI_frameformat[3]))	-- biBitCount

	if AVI_compressor=AVI_NO_COMPRESSION then
		puts(AVI_fn,AVI_DWORD(0))
	elsif AVI_compressor=AVI_CRUNCH then
		puts(AVI_fn,"CRUN")
	elsif AVI_compressor=AVI_LCRUNCH then
		puts(AVI_fn,"CRUN")
	end if

	puts(AVI_fn,AVI_DWORD(AVI_framesize))
	puts(AVI_fn,AVI_DWORD(0))
	puts(AVI_fn,AVI_DWORD(0))
	puts(AVI_fn,AVI_DWORD(0))
	if AVI_compressor=AVI_LCRUNCH then
		puts(AVI_fn,AVI_DWORD(1))
	else
		puts(AVI_fn,AVI_DWORD(0))
	end if
	
	padding = #07F4-where(AVI_fn)
	if padding then
		puts(AVI_fn,"JUNK")
		puts(AVI_fn,AVI_DWORD(padding-8))
		padding = floor((padding-8)/4)	
		for i=1 to padding do
			puts(AVI_fn,{0,0,0,0})
		end for
	end if
	
	puts(AVI_fn,"LIST"&{0,0,0,0}&"movi")

	AVI_framesizes = {}
	AVI_totframesize = 0
	AVI_largestframe = 0
	
	if AVI_compressor=AVI_CRUNCH then
		if not crunch_init_compressor(AVI_frameformat[1],AVI_frameformat[2]) then
			AVI_lasterrormessage = "Failed to initialize compressor"
			return 0
		end if
		compressed = allocate(AVI_framesize)
		if compressed<=0 then
			AVI_lasterrormessage = "Failed to allocate memory for output buffer"
			return 0
		end if
		
	elsif AVI_compressor=AVI_LCRUNCH then
		if not lcrunch_init_compressor(AVI_frameformat[1],AVI_frameformat[2]) then
			AVI_lasterrormessage = "Failed to initialize compressor"
			return 0
		end if
		compressed = allocate(floor(AVI_framesize*2/3))
		if compressed<=0 then
			AVI_lasterrormessage = "Failed to allocate memory for output buffer"
			return 0
		end if
	end if
	
	return 1	
end function




global procedure AVI_End()
	integer indexsize
	integer cumul
	
	if AVI_numframes=0 then
		close(AVI_fn)
		return 
	end if
	
	indexsize = (AVI_numframes*16)+8
	
	-- Write the total filesize	
	if seek(AVI_fn,4) then end if
	puts(AVI_fn,AVI_DWORD(#800 + AVI_totframesize + (AVI_numframes*8) + indexsize))

	-- Set the max transfer rate
	if seek(AVI_fn,#24) then end if
	puts(AVI_fn,AVI_DWORD(AVI_largestframe*AVI_framerate))
	
	if seek(AVI_fn,#30) then end if
	puts(AVI_fn,AVI_DWORD(AVI_numframes))

	-- Set the suggested buffer size
	if seek(AVI_fn,60) then end if
	puts(AVI_fn,AVI_DWORD(AVI_largestframe))
	
	if seek(AVI_fn,#8C) then end if
	puts(AVI_fn,AVI_DWORD(AVI_numframes))

	if seek(AVI_fn,#7F8) then end if
	puts(AVI_fn,AVI_DWORD(AVI_totframesize + (AVI_numframes*8) + 4))

	-- Write the frame index
	cumul = 0
	if seek(AVI_fn,-1) then end if
	puts(AVI_fn,"idx1")
	puts(AVI_fn,AVI_DWORD(indexsize-8))
	for i=0 to AVI_numframes-1 do
		puts(AVI_fn,"00db"&{16,0,0,0})
		if i=0 then
			puts(AVI_fn,AVI_DWORD(4))
		else
			puts(AVI_fn,AVI_DWORD(4 + (i*8) + cumul))
		end if
		puts(AVI_fn,AVI_DWORD(AVI_framesizes[i+1]))
		cumul += AVI_framesizes[i+1]
	end for
	
	AVI_framesizes = {}
	close(AVI_fn)

	if AVI_compressor=AVI_CRUNCH then
		crunch_close_compressor()
		free(compressed)
	elsif AVI_compressor=AVI_LCRUNCH then
		lcrunch_close_compressor()
		free(compressed)
	end if	
end procedure




global function AVI_AddFrame(atom bmBits)
	integer framesize
	atom startt
	
	if AVI_compressor = AVI_NO_COMPRESSION then
		framesize = AVI_framesize
		puts(AVI_fn,"00db"&AVI_DWORD(framesize))
		for i=0 to floor(AVI_framesize/4)-1 do
			puts(AVI_fn,peek({bmBits+(i*4),4}))
		end for
		
	elsif AVI_compressor = AVI_CRUNCH then
		startt = time()
		if not crunch_encode(bmBits,compressed,AVI_frameformat[1],AVI_frameformat[2]) then end if
		AVI_compressiontime = time()-startt
		framesize = peek4u(compressed) + peek4u(compressed+4) + peek4u(compressed+8) + 12
		puts(AVI_fn,"00dc"&AVI_DWORD(framesize))
		for i=0 to framesize-1 do
			puts(AVI_fn,peek(compressed+i))
		end for
		
	elsif AVI_compressor = AVI_LCRUNCH then
		startt = time()
		if not lcrunch_encode(bmBits,compressed,AVI_frameformat[1],AVI_frameformat[2]) then end if
		AVI_compressiontime = time()-startt
		framesize = peek4u(compressed) + peek4u(compressed+4) + peek4u(compressed+8) + 12
		puts(AVI_fn,"00dc"&AVI_DWORD(framesize))
		for i=0 to framesize-1 do
			puts(AVI_fn,peek(compressed+i))
		end for
	end if

	
	if framesize>AVI_largestframe then
		AVI_largestframe = framesize
	end if
	
	AVI_framesizes &= framesize
	AVI_totframesize += framesize
	AVI_numframes += 1
	
	return 1
end function



global function AVI_GetLastFrameSize()
	if AVI_numframes=0 then
		AVI_lasterrormessage = "No frames have been compressed"
		return 0
	end if
	return AVI_framesizes[AVI_numframes]
end function



global function AVI_GetLastCompressionTime()
	if (not AVI_compressor) or AVI_numframes=0 then
		AVI_lasterrormessage = "No frames have been compressed"
		return 0
	end if
	return AVI_compressiontime
end function



global function AVI_GetOutputFramerate()
	return AVI_framerate
end function


------------------------------------------------------------------------------------------------------------------------

-- Default values
AVI_filename = "unknown.avi"
AVI_framerate = 30
AVI_compressor = AVI_NO_COMPRESSION
AVI_lasterrormessage = ""


global function AVI_GetLastErrorMessage()
	return AVI_lasterrormessage
end function

------------------------------------------------------------------------------------------------------------------------


global function AVI_Load(sequence filename)
	sequence s
	integer padding,chsize
	
	AVI_fnIn = open(filename,"rb")
	if AVI_fnIn=-1 then
		AVI_lasterrormessage = "Could not open file: "&filename
		return 0
	end if
	
	s = get_bytes(AVI_fnIn,4)
	if length(s)!=4 or (not equal(s,"RIFF")) then
		AVI_lasterrormessage = "No RIFF tag found"
		close(AVI_fnIn)
		return 0
	end if
	
	s = get_bytes(AVI_fnIn,24)
	AVI_filesizeIn = AVI_DWORD(s[1..4])
	
	if not equal(s[17..24],"hdrlavih") then
		AVI_lasterrormessage = "Main AVI header not found"
		close(AVI_fnIn)
		return 0
	end if

	chsize = AVI_DWORD(get_bytes(AVI_fnIn,4))
	s = get_bytes(AVI_fnIn,chsize)
	AVI_framerateIn = floor(1000000/AVI_DWORD(s[1..4]))
	AVI_numframesIn = AVI_DWORD(s[17..20])
	AVI_suggbuffersize = AVI_DWORD(s[29..32])
	AVI_frameformatIn = {AVI_DWORD(s[33..36]),AVI_DWORD(s[37..40])}
	AVI_streams = AVI_DWORD(s[25..28])
	if AVI_streams>1 then
		AVI_lasterrormessage = sprintf("Found more than 1 stream (%d)",AVI_streams)
		close(AVI_fnIn)
		return 0
	end if
	
	s = get_bytes(AVI_fnIn,24)
	if length(s)!=24 then
		AVI_lasterrormessage = "File error"
		close(AVI_fnIn)
		return 0
	end if
	if not (equal(s[1..4],"LIST") and  equal(s[9..24],"strlstrh"&{#38,0,0,0}&"vids")) then
		AVI_lasterrormessage = "Video stream not found"
		close(AVI_fnIn)
		return 0
	end if
	s = get_bytes(AVI_fnIn,52)
	s = get_bytes(AVI_fnIn,12)
	if not equal(s,"strf"&{#28,0,0,0,#28,0,0,0}) then
		AVI_lasterrormessage = "Bad AVI header"
		close(AVI_fnIn)		
		return 0
	end if
	s = get_bytes(AVI_fnIn,36)
	AVI_frameformatIn &= AVI_WORD(s[11..12])
	if AVI_frameformatIn[3] != 24 then
		AVI_lasterrormessage = sprintf("Unsupported bitcount (%d)",AVI_frameformatIn[3])
		close(AVI_fnIn)
		return 0
	end if
	
	if equal(s[13..16],{0,0,0,0}) then
		AVI_decompressor = AVI_NO_COMPRESSION
	elsif equal(s[13..16],"CRUN") then
		if AVI_DWORD(s[33..36])=0 then
			AVI_decompressor = AVI_CRUNCH
		elsif AVI_DWORD(s[33..36])=1 then
			AVI_decompressor = AVI_LCRUNCH
		else
			AVI_decompressor = AVI_UNKNOWN
		end if
	else
		AVI_decompressor = AVI_UNKNOWN
		AVI_lasterrormessage = "Unknown compressor"
		close(AVI_fnIn)
		return 0
	end if

	-- Estimate a good buffer size if none was suggested
	
	if AVI_suggbuffersize = 0 then
		if AVI_decompressor = AVI_NO_COMPRESSION then
			AVI_suggbuffersize = (AVI_frameformatIn[1]*AVI_frameformatIn[2]*AVI_frameformatIn[3]/8)
		else
			AVI_suggbuffersize = 2*floor(AVI_filesizeIn/AVI_numframesIn)
		end if
	end if

	while 1 do
		s = get_bytes(AVI_fnIn,12)
		if length(s)!=12 then
			AVI_lasterrormessage = "Unexpected end of file"
			close(AVI_fnIn)
			return 0
		end if
		
		if equal(s[1..4],"JUNK") then
			padding = AVI_DWORD(s[5..8])-4
			for i=1 to padding do
				if getc(AVI_fnIn) then end if
			end for
		elsif equal(s[1..4],"LIST") then
			if equal(s[9..12],"movi") then
				exit
			elsif equal(s[9..12],"odml") then
				padding = AVI_DWORD(s[5..8])-4
				for i=1 to padding do
					if getc(AVI_fnIn) then end if
				end for
			end if				
		else
			AVI_lasterrormessage = "Error"
			close(AVI_fnIn)
			return 0
		end if
	end while
	
	AVI_indexpos = where(AVI_fnIn)+AVI_DWORD(s[5..8])-4
	AVI_movipos = where(AVI_fnIn)-4
	
	if AVI_decompressor = AVI_CRUNCH then
		if not crunch_init_decompressor() then
			AVI_lasterrormessage = sprintf("Failed to initialize decompressor (%d)",AVI_decompressor)
			return 0
		end if
		AVI_inputbuffer = allocate(AVI_suggbuffersize)

	elsif AVI_decompressor = AVI_LCRUNCH then
		if not lcrunch_init_decompressor(AVI_frameformatIn[1],AVI_frameformatIn[2]) then
			AVI_lasterrormessage = sprintf("Failed to initialize decompressor (%d)",AVI_decompressor)
			return 0
		end if
		AVI_inputbuffer = allocate(AVI_suggbuffersize)

	end if
	
	AVI_inputpos = 0
	return 1
end function



global function AVI_GetNextFrame(atom dest)
	integer framesize
	sequence s
	
	if AVI_inputpos>=AVI_numframesIn then
		AVI_lasterrormessage = "No more frames"
		return 0
	end if
	s = get_bytes(AVI_fnIn,8)
	if not (equal(s[1..4],"00db") or equal(s[1..4],"00dc")) then
		AVI_lasterrormessage = "Unknown format"
		return 0
	end if
	
	framesize = AVI_DWORD(s[5..8])

	if AVI_decompressor = AVI_NO_COMPRESSION then
		for i=0 to framesize-1 do
			poke(dest+i,getc(AVI_fnIn))
		end for
	elsif AVI_decompressor = AVI_CRUNCH then
		for i=0 to framesize-1 do
			poke(AVI_inputbuffer+i,getc(AVI_fnIn))
		end for
		if not crunch_decode(AVI_inputbuffer,dest) then
			AVI_lasterrormessage = "Decompression failed"
			return 0
		end if

	elsif AVI_decompressor = AVI_LCRUNCH then
		for i=0 to framesize-1 do
			poke(AVI_inputbuffer+i,getc(AVI_fnIn))
		end for
		if not lcrunch_decode(AVI_inputbuffer,dest) then
			AVI_lasterrormessage = "Decompression failed"
			return 0
		end if
	end if
	
	AVI_inputpos += 1
	return 1
end function



global procedure AVI_Unload()
	if AVI_decompressor = AVI_CRUNCH then
		crunch_close_decompressor()
		free(AVI_inputbuffer)
	end if
	close(AVI_fnIn)
end procedure



global function AVI_Seek(integer framenum)
	sequence s
	integer oldpos
	
	if framenum>=AVI_numframesIn then
		AVI_lasterrormessage = "Tried to seek past end of file"
		return -1
	end if
	
	oldpos = where(AVI_fnIn)
	if seek(AVI_fnIn,AVI_indexpos) then
	end if

	s = get_bytes(AVI_fnIn,8)
	if not equal(s[1..4],"idx1") then
		if seek(AVI_fnIn,oldpos) then end if
		AVI_lasterrormessage = "No index found"
		return -1
	end if
	
	if seek(AVI_fnIn,where(AVI_fnIn)+(framenum*16)) then end if
	s = get_bytes(AVI_fnIn,16)
	if seek(AVI_fnIn,AVI_movipos+AVI_DWORD(s[9..12])) then
	end if
	
	return framenum
end function



global function AVI_GetInputFrameCount()
	return AVI_numframesIn
end function


global function AVI_GetInputFrameFormat()
	return AVI_frameformatIn
end function


global function AVI_IsInputCompressed()
	return (AVI_decompressor > AVI_NO_COMPRESSION)
end function


global function AVI_GetDecompressorName()
	return AVI_compressorNames[AVI_decompressor+1]
end function
