; Crunch bitmap encoder
; /Mic, 2002
;


.686p
.MODEL FLAT, STDCALL
OPTION CASEMAP:NONE 		; Case sensitive


include windows.inc
include kernel32.inc
includelib kernel32.lib

IFDEF _DEBUG_
ECHO Including debug code
include user32.inc
includelib user32.lib
wsprintfA PROTO C :DWORD, :VARARG
wsprintf equ <wsprintfA>
ENDIF


;######################## Macros ########################

return MACRO arg
 	mov eax, arg
 	ret
ENDM


RGB24_TO_YUV444 MACRO
	push ecx
	push edx
	movzx eax,word ptr [esi+edi]
	mov ebx,eax 
	movzx ecx,byte ptr [esi+edi+2] 
	shr ebx,8 
	and eax,255 
	movzx edx,byte ptr [R2Y+eax] 
	add dl,[G2Y+ebx]
	add edi,3
	add dl,[B2Y+ecx] 
	add edx,16 
	mov [yuv],edx 
	mov edx,[R2U+eax*4] 
	add edx,[G2U+ebx*4] 
	add edx,[B2U+ecx*4] 
	add edx,128 
	mov [yuv+4],edx 
	mov edx,[R2V+eax*4] 
	add edx,[G2V+ebx*4] 
	add edx,[B2V+ecx*4] 
	add edx,128 
	mov [yuv+8],edx
	pop edx
	pop ecx
ENDM


RGB32_TO_YUV444 MACRO
	push ecx
	push edx
	mov eax,[esi+edi]
	mov ebx,eax
	mov ecx,eax
	shr ebx,8 
	shr ecx,16
	and eax,255
	movzx edx,byte ptr [R2Y+ecx]
	add dl,[G2Y+ebx]
	add edi,4
	add dl,[B2Y+eax] 
	add edx,16 
	mov [yuv],edx 
	mov edx,[R2U+ecx*4] 
	add edx,[G2U+ebx*4] 
	add edx,[B2U+eax*4] 
	add edx,128 
	mov [yuv+4],edx 
	mov edx,[R2V+ecx*4] 
	add edx,[G2V+ebx*4] 
	add edx,[B2V+eax*4] 
	add edx,128 
	mov [yuv+8],edx
	pop edx
	pop ecx
ENDM


;######################### Data #########################
 	
.data

	streams	dd NULL
	TRIGGER	equ 0FEh
	
	extern R2Y:BYTE
	extern G2Y:BYTE
	extern B2Y:BYTE
	extern R2U:DWORD
	extern G2U:DWORD
	extern B2U:DWORD
	extern R2V:DWORD
	extern G2V:DWORD
	extern B2V:DWORD

IFDEF _DEBUG_
	szString		db 48 dup (0)
	szFormatString		db 'Lengths: %d, %d, %d',0
	szError			db 'Info',0
ENDIF	

.data?

	count dd 3 dup (?)
	yuv dd 3 dup (?)
	prevyuv dd 3 dup (?)
	


;######################### Code #########################

.code


ResetStreams PROC
	push ebx
	xor eax,eax
	mov ebx,streams
	mov [ebx],eax
	mov [ebx+4],eax
	mov [ebx+8],eax
	pop ebx
	ret
ResetStreams ENDP



InitStreams PROC bmSize:DWORD
	mov eax,bmSize
	add eax,eax
	add eax,24
	invoke LocalAlloc,LPTR,eax
	.if eax == NULL
		return FALSE
	.endif
	
	mov streams,eax
	invoke ResetStreams
	return TRUE
InitStreams ENDP


ConfStreams PROC dest:DWORD,bmSize:DWORD
	push edi
	
	mov edi,streams
	mov eax,dest
	mov [edi+12],eax
	lea eax,[edi+24]
	mov [edi+16],eax
	add eax,bmSize
	mov [edi+20],eax
	
	pop edi
	ret
ConfStreams ENDP


KillStreams PROC
	.if streams != NULL
		invoke LocalFree, streams
		mov streams,NULL
	.endif
	ret
KillStreams ENDP



StreamWriteByte PROC streamNum:DWORD,value:DWORD
	push eax
	push ebx
	push ecx
	push edx
	push edi
	
	mov edx,streams
	mov ecx,streamNum
	mov ebx,[edx+ecx*4]
	mov eax,value
	inc dword ptr [edx+ecx*4]
	add edx,12
	mov edi,[edx+ecx*4]
	mov [edi+ebx],al
	
	pop edi
	pop edx
	pop ecx
	pop ebx
	pop eax
	ret
StreamWriteByte ENDP



StreamWriteWord PROC streamNum:DWORD,value:DWORD
	push eax
	push ebx
	push ecx
	push edx
	push edi

	mov edx,streams	
	mov ecx,streamNum
	mov ebx,[edx+ecx*4]
	mov eax,value
	mov edi,[edx+ecx*4+12]
	mov [edi+ebx],ax
	add ebx,2
	mov [edx+ecx*4],ebx
	
	pop edi
	pop edx
	pop ecx
	pop ebx
	pop eax
	ret
StreamWriteWord ENDP



StreamWriteRepCode PROC streamNum:DWORD,escape:DWORD,repeats:DWORD,value:DWORD
	push eax
	push ebx
	
	mov eax,repeats
	mov ebx,escape
	shl eax,8
	or ebx,eax
	invoke StreamWriteWord,streamNum,ebx
	
	mov eax,value
	invoke StreamWriteByte,streamNum,eax
	
	pop ebx
	pop eax
	ret
StreamWriteRepCode ENDP

	



; Crunch RGB24 and RGB32 data
;
Crunch_RGB PROC bmBits:DWORD,bmSize:DWORD,inType:DWORD
	pusha 
	mov ebx,streams 
	xor eax,eax 
	mov [ebx],eax 			; Reset streams
	mov [ebx+4],eax 
	mov [ebx+8],eax 
	mov edi,eax 			; edi = source offset
	mov ecx,bmSize			;  
	mov esi,bmBits			; Pointer to source bitmap 
	@@CRGB_main_loop:
		
		.if inType == 3
			RGB24_TO_YUV444
		.else
			RGB32_TO_YUV444
		.endif
	
		.if edi < 5		
			mov eax,1 
			mov [count],eax 
			mov [count+4],eax 
			mov [count+8],eax 
			mov ebx,esi 
			mov edx,ecx 
			push edi
			mov esi,OFFSET yuv 
			mov edi,OFFSET prevyuv 
			mov ecx,3 
			rep movsd
			pop edi
			mov esi,ebx 
			mov ecx,edx 
			jmp @@CRGB_next_pixel 
		.endif 
	
		push ecx 
		mov ecx,2 
		@@CRGB_yuv_loop: 
			mov eax,[yuv+ecx*4]
			mov ebx,[prevyuv+ecx*4]
			.if eax == ebx
				inc dword ptr [count+ecx*4] 
				mov eax,[count+ecx*4] 
				.if eax == 255
					;mov ebx,[yuv+ecx*4] 
					invoke StreamWriteRepCode,ecx,TRIGGER,eax,ebx
					xor ebx,ebx
					mov [count+ecx*4],ebx 
				.endif

			.else
				mov eax,[count+ecx*4] 
				.if eax < 3
					.if eax == 0
						jmp @@CRGB_end_else
					.endif
				
					mov ebx,[prevyuv+ecx*4] 
					.if ebx == TRIGGER
						@@CRGB_output_loop1: 
							invoke StreamWriteWord,ecx,ebx
							dec eax 
						jnz @@CRGB_output_loop1 
					.else
						@@CRGB_output_loop2: 
							invoke StreamWriteByte,ecx,ebx
							dec eax 
						jnz @@CRGB_output_loop2 
					.endif
				.else	
					mov ebx,[prevyuv+ecx*4] 
					invoke StreamWriteRepCode,ecx,TRIGGER,eax,ebx
				.endif
				@@CRGB_end_else:
				mov eax,1 
				mov [count+ecx*4],eax 
				mov eax,[yuv+ecx*4] 
				mov [prevyuv+ecx*4],eax 
			.endif
			@@CRGB_next_component: 
			dec ecx 
		jns @@CRGB_yuv_loop 
		pop ecx
	@@CRGB_next_pixel: 
	dec ecx 
	jnz @@CRGB_main_loop
	popa
	ret
Crunch_RGB ENDP



; Crunch interleaved YUV data, such as YUY2 or UYVY
;
Crunch_interleaved_YUV PROC bmBits:DWORD,bmSize:DWORD,inType:DWORD
	local ecxmod4:DWORD
	
	pusha 
	mov ebx,streams 
	xor eax,eax 
	mov [ebx],eax 			; Reset streams
	mov [ebx+4],eax 
	mov [ebx+8],eax 
	mov edi,eax 			; edi = source offset
	mov eax,-1
	mov [prevyuv],eax
	mov [prevyuv+4],eax
	mov [prevyuv+8],eax
	mov ecx,bmSize			;  
	lea ecx,[ecx+ecx]
	mov esi,bmBits			; Pointer to source bitmap 
	mov ecxmod4,0
	@@CYUV_main_loop:
		.if inType == 6
			.if edi == 0
				mov edx,1
			.elseif edi == 2
				mov edx,2
			.else
				mov edx,0
			.endif
		.else
			.if edi == 1
				mov edx,1
			.elseif edi == 3
				mov edx,2
			.else
				mov edx,0
			.endif
		.endif
		
		mov eax,[prevyuv+edx*4]
		.if eax == -1		
			mov eax,1 
			mov [count+edx*4],eax 
			movzx eax,byte ptr [esi]
			mov [prevyuv+edx*4],eax
			jmp @@CYUV_next_pixel 
		.endif 
	
		movzx eax,byte ptr [esi] 
		mov ebx,[prevyuv+edx*4] 
		cmp eax,ebx
		jne @@CYUV_no_repeat 
			inc dword ptr [count+edx*4] 
			mov eax,[count+edx*4] 
			.if eax == 255
				movzx ebx,byte ptr [esi] 
				invoke StreamWriteRepCode,edx,TRIGGER,eax,ebx
				xor ebx,ebx
				mov [count+edx*4],ebx 
			.endif
			jmp @@CYUV_next_component 

		@@CYUV_no_repeat: 
		mov eax,[count+edx*4] 
		.if eax < 3
			.if eax == 0
				jmp @@CYUV_end_else
			.endif
			
			mov ebx,[prevyuv+edx*4] 
			.if ebx == TRIGGER
				@@CYUV_output_loop1: 
					invoke StreamWriteWord,edx,ebx
					dec eax 
				jnz @@CYUV_output_loop1 
			.else
				@@CYUV_output_loop2: 
					invoke StreamWriteByte,edx,ebx
					dec eax 
				jnz @@CYUV_output_loop2 
			.endif
		.else	
			mov ebx,[prevyuv+edx*4] 
			invoke StreamWriteRepCode,edx,TRIGGER,eax,ebx
		.endif
		@@CYUV_end_else:
		mov eax,1 
		mov [count+edx*4],eax 
		movzx eax,byte ptr [esi] 
		mov [prevyuv+edx*4],eax 
		@@CYUV_next_component: 
	
	@@CYUV_next_pixel: 
	inc esi
	xor eax,eax
	inc edi
	cmp edi,4
	cmovge edi,eax
	
	dec ecx 
	jnz @@CYUV_main_loop
	popa
	ret
Crunch_interleaved_YUV ENDP



Crunch PROC bmBits:DWORD,dest:DWORD,bmSize:DWORD,inType:DWORD
	local framesize:DWORD
	
	pusha
	cld
	
	.if inType < 5
		invoke Crunch_RGB, bmBits,bmSize,inType
	.else
		invoke Crunch_interleaved_YUV, bmBits,bmSize,inType
	.endif
	
	
	; Flush any remaining chars
	
	mov ecx,2					; Start with stream 2 (i.e. V) 
	@@C_loop_streams: 
		mov eax,[count+ecx*4]
		.if eax == 0
			jmp @@C_next_stream		; Do nothing if there are no remaining characters
		.endif
		
		.if eax < 3
			mov ebx,[prevyuv+ecx*4] 
			.if ebx == TRIGGER
				@@C_output_TRIGGERS: 
					invoke StreamWriteWord,ecx,ebx
					dec eax 
				jnz @@C_output_TRIGGERS 
			.else 
				@@C_output_bytes:
					invoke StreamWriteByte,ecx,ebx
					dec eax 
				jnz @@C_output_bytes 
			.endif
		.else
			mov ebx,[prevyuv+ecx*4] 
			invoke StreamWriteRepCode,ecx,TRIGGER,eax,ebx
		.endif
		@@C_next_stream: 
		dec ecx 
	jns @@C_loop_streams 

	mov ebx,streams 
	mov eax,[ebx] 
	mov edx,[ebx+8] 
	mov ebx,[ebx+4] 
	
	IFDEF _DEBUG_
	pusha
	invoke wsprintf, ADDR szString,ADDR szFormatString,eax,ebx,edx
	invoke MessageBox, NULL,ADDR szString,ADDR szError,MB_ICONINFORMATION
	popa
	ENDIF
	
	mov edi,dest 
	mov [edi],eax 
	mov [edi+4],ebx 
	mov [edi+8],edx 
	mov ecx,eax 
	add ecx,ebx 
	add ecx,edx 
	mov framesize,ecx
	and ecx,3 
	add edx,ecx 
	add framesize,ecx
	mov [edi+8],edx 
	
	lea edi,[edi+eax+12] 
	mov esi,streams 
	mov esi,[esi+16]
	mov ecx,ebx 
	rep movsb 
	mov esi,streams 
	mov esi,[esi+20]
	mov ecx,edx 
	rep movsb 
	
	add framesize,12
	
	popa 
	return framesize
Crunch ENDP


END
