;############################## CRUNCHYUV.ASM ############################
;
;
; /Mic, 2002
; stabmaster_@hotmail.com | http://www.cyd.liu.se/~micol972/site/
;
;
; Compile with MASM:
;
;   ml /c /coff crunchyuv.asm
;   ml /c /coff drvproc.asm
;   ml /c /coff compressor.asm
;   ml /D_USE_CMOV_ /c /coff decompressor.asm
;
; Then, to link the dll:
;
;   link /SUBSYSTEM:WINDOWS /DLL /DEF:crunchyuv.def /OUT:crunchyuv.dll drvproc.obj crunchyuv.obj
;
;
;
;######################### Assembler directives ###########################


.686p
.MODEL FLAT, STDCALL
OPTION CASEMAP:NONE 		; Case sensitive



;############################# Includes ###################################

    include windows.inc
    include kernel32.inc
    include user32.inc
    include .\vfw32.inc
 
    includelib user32.lib
    includelib kernel32.lib

    
;############################### Macros ##################################

 	return MACRO arg
 		mov eax, arg
 		ret
 	ENDM


;############################# Prototypes ################################

	CodecEndCompress PROTO
	InitStreams PROTO :DWORD
	ConfStreams PROTO :DWORD,:DWORD
	ResetStreams PROTO
	KillStreams PROTO
	Crunch PROTO :DWORD,:DWORD,:DWORD,:DWORD

	CodecEndDecompress PROTO
	AllocYUV411Buffer PROTO :DWORD
	FreeYUV411Buffer PROTO
	DeCrunch PROTO :DWORD,:DWORD,:DWORD,:DWORD

        wsprintfA PROTO C :DWORD, :VARARG
        wsprintf equ <wsprintfA>

        
;################################# Data ###################################

.data
	szName			db 'CrunchYUV',0,0	; Name of the codec
	szDescription		db 'CrunchYUV v1.1',0,0	; Description of the codec
	crap			dd 0			; Just four bytes of cr*p
	temp			dd 0			; All purpose variable
	method			dd 0
	
	compressing		dd FALSE		; Compressing-flag
	decompressing		dd FALSE		 

	FOURCC_CRUN		equ 'NURC'		; The FOURCC of this codec, backwards.. (so it's really CRUN)
	FOURCC_YUY2		equ '2YUY'
	FOURCC_UYVY		equ 'YVYU'
	FOURCC_DIB		equ ' BID'
	
	CRUNCHYUV_VERSION	equ 0101h		; Codec version number

	SIZEOF_BITMAPINFOHEADER	equ 40
	TRIGGER			equ 0FEh		; Used by the Crunch codec

	rymul			REAL8 0.256788
	gymul			REAL8 0.504129
	bymul			REAL8 0.097906
	rumul			REAL8 -0.148223
	gumul			REAL8 -0.290993
	bumul			REAL8 0.439216
	rvmul			REAL8 0.439216
	gvmul			REAL8 -0.367788
	bvmul			REAL8 -0.071427
	
	szError			db 'Error',0
	szBadInput		db 'Bad input format',0
	IFDEF _DEBUG_
	ECHO Assembling debug version
	szCantDecompress	db 'Cannot decompress bitmap',0
	szBitCountError		db 'Bitcount must be 24',0
	szInfo			db 'In.bc=%d, Out.bc=%d',0,0,0,0,0
	szFormatInfo		db 'In.bc=%d, Out.bc=%d',0
	ENDIF

	szFourcc		db 28 dup (0)
	szFormatFourcc		db 'FOURCC = 0x%08x',0
	szError2		db 'Error',0
	

.data?
	tagYUV STRUCT
		Y	dd ?
		U	dd ?
		V	dd ?
	tagYUV ENDS
	
	; Color conversion tables
	PUBLIC R2Y
	PUBLIC G2Y
	PUBLIC B2Y
	PUBLIC R2U
	PUBLIC G2U
	PUBLIC B2U
	PUBLIC R2V
	PUBLIC G2V
	PUBLIC B2V
	R2Y	db 256 dup (?)
	G2Y	db 256 dup (?)
	B2Y	db 256 dup (?)
	R2U	dd 256 dup (?)
	G2U	dd 256 dup (?)
	B2U	dd 256 dup (?)
	R2V	dd 256 dup (?)
	G2V	dd 256 dup (?)
	B2V	dd 256 dup (?)
	
	
;##########################################################################

.code

; 0=unknown, 1=CrunchYUV, 3=RGB 24-bit, 4=RGB 32-bit, 5=YUY2, 6=UYVY
GetBitmapType PROC lpbi:DWORD
	.if lpbi == NULL
		return 0
	.endif
	
	push esi
	push ebx

	mov esi,lpbi
	mov eax,[esi+16]		; Get the fourcc (actually biCompression)
	movzx ebx,word ptr [esi+14]	; biBitCount

		;invoke wsprintf, ADDR szFourcc,ADDR szFormatFourcc,eax
		;invoke MessageBox, NULL,ADDR szFourcc,ADDR szError2,MB_ICONERROR	

	.if eax == FOURCC_DIB		
		mov eax,0
	.endif
	.if eax == 0 
		.if ebx == 24
			pop ebx
			pop esi
			return 3
		.elseif ebx == 32
			pop ebx
			pop esi
			return 4
		.endif
		pop ebx
		pop esi
		return 0

	.elseif eax == FOURCC_CRUN
		pop ebx
		pop esi
		return 1

	.elseif eax == FOURCC_YUY2
		pop ebx
		pop esi
		return 5

	.elseif eax == FOURCC_UYVY
		pop ebx
		pop esi
		return 6
	.endif
	
	pop ebx
	pop esi
	return 0
GetBitmapType ENDP	



; Set up color conversion tables
;
InitTables PROC
	local counter:DWORD
	
	pusha
	mov counter,0
	mov ecx,0
	@@IT_loop:
		fild dword ptr [counter]
		fmul qword ptr [rymul]
		fistp dword ptr [temp]
		mov eax,temp
		mov [R2Y+ecx],al
		fild dword ptr [counter]
		fmul qword ptr [gymul]
		fistp dword ptr [temp]
		mov eax,temp
		mov [G2Y+ecx],al
		fild dword ptr [counter]
		fmul qword ptr [bymul]
		fistp dword ptr [temp]
		mov eax,temp
		mov [B2Y+ecx],al
		fild dword ptr [counter]
		fmul qword ptr [rumul]
		fistp dword ptr [R2U+ecx*4]
		fild dword ptr [counter]
		fmul qword ptr [gumul]
		fistp dword ptr [G2U+ecx*4]
		fild dword ptr [counter]
		fmul qword ptr [bumul]
		fistp dword ptr [B2U+ecx*4]
		fild dword ptr [counter]
		fmul qword ptr [rvmul]
		fistp dword ptr [R2V+ecx*4]
		fild dword ptr [counter]
		fmul qword ptr [gvmul]
		fistp dword ptr [G2V+ecx*4]
		fild dword ptr [counter]
		fmul qword ptr [bvmul]
		fistp dword ptr [B2V+ecx*4]
		
		inc ecx
		inc dword ptr counter
		cmp ecx,256
	jne @@IT_loop
	popa
	ret
InitTables ENDP
	


;################################ Codec initialization and whatnot ################################

CodecOpen PROC icinfo:DWORD
	push esi
	.if icinfo != NULL
		mov esi,icinfo
		mov eax,[esi+4]
		.if eax != ICTYPE_VIDEO
			pop esi
			return NULL
		.endif
	.endif
	
	mov eax,ICERR_OK
	mov [esi+20],eax	; Set dwError to ICERR_OK
	
	invoke InitTables

	pop esi
	; I don't use any data that needs to be passed between the driver and the calling application,
	; so I just return a pointer to some variable.
	return OFFSET crap
CodecOpen ENDP



CodecClose PROC codecData:DWORD
	return 1
CodecClose ENDP



; Return info about the codec to the calling application
;
CodecGetInfo PROC icinfo:DWORD,dwSize:DWORD
	.if icinfo == NULL
		return SIZEOF_ICINFO
	.elseif dwSize < SIZEOF_ICINFO
		return 0
	.endif

	push edi	
	mov edi,icinfo
	
	mov eax,SIZEOF_ICINFO
	mov [edi],eax
	mov eax,ICTYPE_VIDEO
	mov [edi+4],eax
	mov eax,FOURCC_CRUN
	mov [edi+8],eax
	mov eax,0
	mov [edi+12],eax
	mov eax,CRUNCHYUV_VERSION
	mov [edi+16],eax
	mov eax,ICVERSION
	mov [edi+20],eax
	
	add edi,24
	invoke MultiByteToWideChar,CP_ACP,0,ADDR szName,-1,edi,16
	add edi,32
	invoke MultiByteToWideChar,CP_ACP,0,ADDR szDescription,-1,edi,128
	
	pop edi
	return SIZEOF_ICINFO
CodecGetInfo ENDP



;################################## Compression-message handlers ##################################


CanCompress PROC lpbiIn:DWORD
	invoke GetBitmapType,lpbiIn
		
		;pusha
		;invoke wsprintf, ADDR szFourcc,ADDR szFormatFourcc,eax
		;invoke MessageBox, NULL,ADDR szFourcc,ADDR szError2,MB_ICONERROR	
		;popa
		
	.if eax == 3
		return TRUE
	.elseif eax == 4
		return FALSE
	.elseif eax == 5
		return TRUE
	.elseif eax == 6
		return TRUE
	.endif
	
	return FALSE
CanCompress ENDP



; Can we compress the input bitmap into the output bitmap?
; If yes, return ICERR_OK, and ICERR_BADFORMAT otherwise
;
CodecQueryCompress PROC lpbiIn:DWORD,lpbiOut:DWORD
	invoke CanCompress,lpbiIn ;,lpbiOut
	.if eax == TRUE
		return ICERR_OK
	.endif
	
	return ICERR_BADFORMAT
CodecQueryCompress ENDP



CodecGetCompressFormat PROC lpbiIn:DWORD,lpbiOut:DWORD
	invoke CanCompress,lpbiIn
	.if eax == FALSE
		;IFDEF _DEBUG_
		invoke MessageBox,NULL,ADDR szBadInput,ADDR szError,MB_ICONERROR		
		;ENDIF
		return ICERR_BADFORMAT
	.endif

	.if lpbiOut == NULL
		return SIZEOF_BITMAPINFOHEADER
	.endif
	
	
	push ecx
	push esi
	push edi
	
	mov ecx,SIZEOF_BITMAPINFOHEADER
	mov esi,lpbiIn
	mov edi,lpbiOut
	rep movsb			; Copy..

	mov esi,lpbiIn
	mov edi,lpbiOut
	
	mov eax,SIZEOF_BITMAPINFOHEADER
	mov ecx,1
	mov [edi],eax			; lpbiOut->biSize
	mov [edi+12],cx			; lpbiOut->biPlanes
	
	invoke GetBitmapType, lpbiIn
	.if eax == 3
		xor eax,eax
		mov [edi+36],eax
		mov eax,24
		mov ecx,FOURCC_CRUN
		mov [edi+14],ax		; lpbiOut->biBitCount
		mov [edi+16],ecx	; lpbiOut->biCompression
		mov eax,[esi+4]		; lpbiIn->biWidth
		mul dword ptr [esi+8]	; lpbiIn->biHeight
		lea eax,[eax+eax*2]	; eax *= 3
		mov [edi+20],eax	; lpbiOut->biSizeImage

	.elseif eax == 5
		mov eax,2
		mov [edi+36],eax
		mov eax,24
		mov ecx,FOURCC_CRUN
		mov [edi+14],ax		; lpbiOut->biBitCount
		mov [edi+16],ecx	; lpbiOut->biCompression
		mov eax,[esi+4]		; lpbiIn->biWidth
		mul dword ptr [esi+8]	; lpbiIn->biHeight
		lea eax,[eax+eax*2]	; eax *= 3
		mov [edi+20],eax	; lpbiOut->biSizeImage

	.elseif eax == 6
		mov eax,2
		mov [edi+36],eax
		mov eax,24
		mov ecx,FOURCC_CRUN
		mov [edi+14],ax		; lpbiOut->biBitCount
		mov [edi+16],ecx	; lpbiOut->biCompression
		mov eax,[esi+4]		; lpbiIn->biWidth
		mul dword ptr [esi+8]	; lpbiIn->biHeight
		lea eax,[eax+eax*2]	; eax *= 3
		mov [edi+20],eax	; lpbiOut->biSizeImage
	.endif
	
	pop edi
	pop esi
	pop ecx
	
	return ICERR_OK
CodecGetCompressFormat ENDP



CodecGetCompressSize PROC lpbiIn:DWORD,lpbiOut:DWORD
	push esi
	
	mov esi,lpbiIn
	mov eax,[esi+4]
	mul dword ptr [esi+8]
	lea eax,[eax+eax*2]
	pop esi
	ret
CodecGetCompressSize ENDP

	

CodecBeginCompress PROC lpbiIn:DWORD,lpbiOut:DWORD
	invoke CodecEndCompress
	
	invoke CanCompress, lpbiIn ;,lpbiOut
	.if eax == FALSE
		return ICERR_BADFORMAT
	.endif

	push esi
	mov esi,lpbiIn

	mov eax,[esi+4]
	mul dword ptr [esi+8]
	invoke InitStreams, eax
	.if eax == TRUE
		mov compressing,TRUE
		pop esi
		return ICERR_OK
	.endif
	
	mov compressing,FALSE
	pop esi
	return ICERR_BADFORMAT
CodecBeginCompress ENDP



CodecCompress PROC iccompress:DWORD,dwSize:DWORD
  	push esi
  	push edi
  	push ebx

  	invoke ResetStreams
	
   	mov esi,iccompress
  	mov eax,[esi+12]
  	
  	mov edi,eax
  	mov eax,[edi+4] 
  	mul dword ptr [edi+8]
  	mov ebx,eax

	mov eax,[esi+8]		; lpOutput
	add eax,12
	invoke ConfStreams, eax,ebx
	
  	mov eax,[esi+12]	; lpbiIn
  	invoke GetBitmapType,eax
  	
  	IFDEF _DEBUG_
  	.if crap == 0
		pusha
	  	mov eax,[esi+12]
	  	mov eax,[eax+20]
		invoke wsprintf, ADDR szFourcc,ADDR szFormatFourcc,eax
		invoke MessageBox, NULL,ADDR szFourcc,ADDR szError2,MB_ICONERROR	
		inc crap
		popa
	.endif
	ENDIF
	
  	mov edi,[esi+8]		; lpOutput
  	mov esi,[esi+16]	; lpInput
  
   	invoke Crunch, esi,edi,ebx,eax

   	mov esi,iccompress
  	mov edi,[esi+4]
  	mov [edi+20],eax	; Store the size of the compressed frame
     	
    	pop ebx
    	pop edi
    	pop esi
	return ICERR_OK
CodecCompress ENDP



CodecEndCompress PROC
	invoke KillStreams
	mov compressing,FALSE
	return ICERR_OK
CodecEndCompress ENDP






;############################### Decompression-message handlers ###############################


; Can we decompress the input bitmap? (only return true if it was compressed with CrunchYUV)
;
CanDecompress PROC lpbiIn:DWORD
	invoke GetBitmapType,lpbiIn
	.if eax < 0
		IFDEF _DEBUG_
		invoke MessageBox,NULL,ADDR szCantDecompress,ADDR szError,MB_ICONERROR
		ENDIF
		return FALSE
	.elseif eax == 1
		return TRUE
	.endif
	return FALSE
CanDecompress ENDP



CanDecompress2 PROC lpbiIn:DWORD,lpbiOut:DWORD
	.if lpbiOut == NULL
		invoke CanDecompress,lpbiIn
		ret
	.endif
	
	push esi
	push edi
	push ebx
	
	mov esi,lpbiIn
	mov edi,lpbiOut
	
	mov eax,[esi+4]			; lpbiIn->biWidth
	mov ebx,[esi+8]			; lpbiIn->biHeight
	cmp eax,[edi+4]
	jne @@CD2_error
	cmp ebx,[edi+8]
	jne @@CD2_error

	invoke CanDecompress,lpbiIn
	.if eax == FALSE
		jmp @@CD2_error
	.endif
	movzx eax,word ptr [edi+14]	; lpbiOut->biBitCount
	.if eax != 24
		IFDEF _DEBUG_
		invoke MessageBox,NULL,ADDR szBitCountError,ADDR szError,MB_ICONERROR
		ENDIF
		jmp @@CD2_error
	.endif
	
	pop ebx
	pop edi
	pop esi
	return TRUE			; Everything seems to be OK
	
	@@CD2_error:
	pop ebx
	pop edi
	pop esi
	return FALSE
CanDecompress2 ENDP



; Can we decompress the input bitmap into the output bitmap?
; If yes, return ICERR_OK, and ICERR_BADFORMAT otherwise
;
CodecQueryDecompress PROC lpbiIn:DWORD,lpbiOut:DWORD
	invoke CanDecompress2,lpbiIn,lpbiOut
	.if eax == TRUE
		return ICERR_OK
	.endif
	
	IFDEF _DEBUG_
	push esi
	push edi
	push ebx
	mov esi,lpbiIn
	mov edi,lpbiOut
	movzx eax,word ptr [esi+14]
	movzx ebx,word ptr [edi+14]
	invoke wsprintf, ADDR szInfo,ADDR szFormatInfo,eax,ebx
	invoke MessageBox, NULL,ADDR szInfo,ADDR szError,MB_ICONERROR	
	pop ebx
	pop edi
	pop esi
	ENDIF
	
	return ICERR_BADFORMAT
CodecQueryDecompress ENDP



CanDecompressEx PROC lpicdEx:DWORD
	push esi
	push ebx
	
	mov esi,lpicdEx
	mov eax,[esi+4]		; lpbiSrc
	mov ebx,[esi+12]	; lpbiDst
	invoke CanDecompress2,eax,ebx
	pop ebx
	pop esi
	ret
CanDecompressEx ENDP



CodecQueryDecompressEx PROC lpicdEx:DWORD
	invoke CanDecompressEx, lpicdEx
	.if eax == TRUE
		return ICERR_OK
	.endif
	
	return ICERR_BADFORMAT
CodecQueryDecompressEx ENDP



CodecGetDecompressFormat PROC lpbiIn:DWORD,lpbiOut:DWORD
	.if lpbiOut == NULL
		return SIZEOF_BITMAPINFOHEADER
	.endif
	
	invoke CanDecompress,lpbiIn
	.if eax == FALSE
		IFDEF _DEBUG_
		invoke MessageBox,NULL,ADDR szBadInput,ADDR szError,MB_ICONERROR		
		ENDIF
		return ICERR_BADFORMAT
	.endif
	
	push ecx
	push esi
	push edi
	
	mov ecx,SIZEOF_BITMAPINFOHEADER
	mov esi,lpbiIn
	mov edi,lpbiOut
	rep movsb			; Copy..

	mov esi,lpbiIn
	mov edi,lpbiOut
	
	mov eax,SIZEOF_BITMAPINFOHEADER
	mov ecx,1
	mov [edi],eax			; lpbiOut->biSize
	mov [edi+12],cx			; lpbiOut->biPlanes
	
	invoke GetBitmapType, lpbiIn
	.if eax == 1			; 1 = CrunchYUV
		mov eax,24
		mov ecx,0
		mov [edi+14],ax		; lpbiOut->biBitCount
		mov [edi+16],ecx	; lpbiOut->biCompression
		mov eax,[esi+4]		; lpbiIn->biWidth
		mul dword ptr [esi+8]	; lpbiIn->biHeight
		lea eax,[eax+eax*2]	; eax *= 3
		mov [edi+20],eax	; lpbiOut->biSizeImage
		mov eax,[esi+36]
		mov method,eax
	.endif
	
	pop edi
	pop esi
	pop ecx
	
	return ICERR_OK
CodecGetDecompressFormat ENDP



CodecBeginDecompress PROC lpbiIn:DWORD,lpbiOut:DWORD
	invoke CodecEndDecompress
	
	invoke CanDecompress2, lpbiIn,lpbiOut
	.if eax == FALSE
		return ICERR_BADFORMAT
	.endif

	pusha
	mov esi,lpbiIn
	mov eax,[esi+36]
	.if eax == 1		; biClrImportant = 1 signals the input data is in YUV411 format

		mov eax,[esi+4]
		mul dword ptr [esi+8]
		invoke AllocYUV411Buffer, eax
		.if eax == FALSE
			popa
			return ICERR_BADFORMAT
		.endif
		mov method,1
	.endif
	
	mov decompressing,TRUE

	return ICERR_OK
CodecBeginDecompress ENDP



CodecDecompress PROC icdecompress:DWORD,dwSize:DWORD
  	push esi
  	push edi
  	push ebx
  	
  	.if decompressing == FALSE
		mov esi,icdecompress
	    	mov eax,[esi+4]
	    	mov ebx,[esi+12]
	    	invoke CodecBeginDecompress, eax,ebx
	    	.if eax != ICERR_OK
	      		pop ebx
			pop edi
			pop esi
	      		ret
	    	.endif
	.endif    	
  
  	mov esi,icdecompress
  	mov eax,[esi+12]
  	movzx ebx,word ptr [eax+14]
  	.if ebx != 24
  		IFDEF _DEBUG_
	  	invoke MessageBox, NULL,ADDR szBitCountError,ADDR szError,MB_ICONERROR
  		ENDIF
  		return ICERR_BADFORMAT
  	.endif
  	
  	mov edi,eax
  	mov eax,[edi+4] 
  	mul dword ptr [edi+8]
  	mul word ptr [edi+14]
  	shr eax,3
  	mov [edi+20],eax
  	
  	mov eax,[esi+4]
  	mov eax,[eax+4]
  	
  	mov edi,[esi+16]
  	mov esi,[esi+8]
  	
    	invoke DeCrunch, esi,edi,eax,method
    	
    	pop ebx
    	pop edi
    	pop esi
	return ICERR_OK
CodecDecompress ENDP



CodecEndDecompress PROC
	invoke FreeYUV411Buffer
	mov decompressing,FALSE
	return ICERR_OK
CodecEndDecompress ENDP





END 
