-- Crunch bitmap codec (lossless version)
-- /Mic, 2002
--
-- Transforms the image from RGB24 to YUV before encoding
-- Uses the crunch algorithm, which is (c) 1987 Andrew Binstock
--
-- Updated 20021008
--  * Fixed a bug in the compressor that caused it to crash in some cases
--
-- Updated 20021001
--  * Rewrote the encoder in assembly -> much faster
--  * Added a decoder
--


include machine.e


atom streams,stream_data
atom crunch,crunch_data,rgb2yuv,stream_write_byte,stream_write_code
atom decrunch,decrunch_data



function init_streams(integer bmSize)
	streams = allocate(24+(bmSize*3)+16)
	if streams<=0 then 
		return 0
	end if
	stream_data = streams+24+(bmSize*3)
	poke4(streams,{0,0,0,0,bmSize,bmSize*2})	
	return 1
end function



procedure reset_streams()
	poke4(streams,{0,0,0})
end procedure



procedure kill_streams()
	free(streams)
end procedure




constant TRIGGER = #FE


global function crunch_init_compressor(integer bmWidth,integer bmHeight)
	if not init_streams(bmWidth*bmHeight) then
		return 0
	end if
	reset_streams()

	crunch_data = allocate(128)
	if crunch_data<=0 then
		return 0
	end if
	
	stream_write_byte = allocate(43)
	poke(stream_write_byte, {
	#60,#8B,#0D,#00,#00,#00,#00,#B8,#00,#00,#00,#00,#8B,#14,#88,#89,#D5,#45,#89,#2C,
	#88,#83,#C0,#0C,#8B,#1C,#88,#01,#D8,#83,#C0,#0C,#8B,#1D,#00,#00,#00,#00,#88,#1C,
	#02,#61,#C3})
	poke4(stream_write_byte + 3, stream_data)
	poke4(stream_write_byte + 8, streams)
	poke4(stream_write_byte + 34, stream_data+4)

	stream_write_code = allocate(56)
	poke(stream_write_code, {
	#60,#BB,#00,#00,#00,#00,#8B,#2D,#00,#00,#00,#00,#A1,#00,#00,#00,#00,#A3,#00,#00,
	#00,#00,#BB,#00,#00,#00,#00,#FF,#D3,#A1,#00,#00,#00,#00,#A3,#00,#00,#00,#00,#BB,
	#00,#00,#00,#00,#FF,#D3,#89,#2D,#00,#00,#00,#00,#FF,#D3,#61,#C3})
	poke4(stream_write_code + 2, stream_write_byte)
	poke4(stream_write_code + 8, stream_data+4)
	poke4(stream_write_code + 13, stream_data+12)
	poke4(stream_write_code + 18, stream_data+4)
	poke4(stream_write_code + 23, stream_write_byte)
	poke4(stream_write_code + 30, stream_data+8)
	poke4(stream_write_code + 35, stream_data+4)
	poke4(stream_write_code + 40, stream_write_byte)
	poke4(stream_write_code + 48, stream_data+4)


	rgb2yuv = allocate(219)
	if rgb2yuv<=0 then
		return 0
	end if
	poke(rgb2yuv, {
	#60,#DB,#05,#00,#00,#00,#00,#D8,#0D,#00,#00,#00,#00,#D8,#05,#00,#00,#00,#00,#D9,
	#1D,#00,#00,#00,#00,#DB,#05,#00,#00,#00,#00,#D8,#0D,#00,#00,#00,#00,#D8,#05,#00,
	#00,#00,#00,#D9,#1D,#00,#00,#00,#00,#DB,#05,#00,#00,#00,#00,#D8,#0D,#00,#00,#00,
	#00,#D8,#05,#00,#00,#00,#00,#DB,#1D,#00,#00,#00,#00,#DB,#05,#00,#00,#00,#00,#D8,
	#0D,#00,#00,#00,#00,#D8,#05,#00,#00,#00,#00,#D9,#1D,#00,#00,#00,#00,#DB,#05,#00,
	#00,#00,#00,#D8,#0D,#00,#00,#00,#00,#D8,#05,#00,#00,#00,#00,#D9,#1D,#00,#00,#00,
	#00,#DB,#05,#00,#00,#00,#00,#D8,#0D,#00,#00,#00,#00,#D8,#05,#00,#00,#00,#00,#DB,
	#1D,#00,#00,#00,#00,#DB,#05,#00,#00,#00,#00,#D8,#0D,#00,#00,#00,#00,#D8,#05,#00,
	#00,#00,#00,#D9,#1D,#00,#00,#00,#00,#DB,#05,#00,#00,#00,#00,#D8,#0D,#00,#00,#00,
	#00,#D8,#05,#00,#00,#00,#00,#D9,#1D,#00,#00,#00,#00,#DB,#05,#00,#00,#00,#00,#D8,
	#0D,#00,#00,#00,#00,#D8,#05,#00,#00,#00,#00,#DB,#1D,#00,#00,#00,#00,#61,#C3})
	poke4(rgb2yuv + 3, crunch_data)
	poke4(rgb2yuv + 9, crunch_data+24)
	poke4(rgb2yuv + 15, crunch_data+60)
	poke4(rgb2yuv + 21, crunch_data+12)
	poke4(rgb2yuv + 27, crunch_data+4)
	poke4(rgb2yuv + 33, crunch_data+36)
	poke4(rgb2yuv + 39, crunch_data+12)
	poke4(rgb2yuv + 45, crunch_data+12)
	poke4(rgb2yuv + 51, crunch_data+8)
	poke4(rgb2yuv + 57, crunch_data+48)
	poke4(rgb2yuv + 63, crunch_data+12)
	poke4(rgb2yuv + 69, crunch_data+12)
	poke4(rgb2yuv + 75, crunch_data)
	poke4(rgb2yuv + 81, crunch_data+28)
	poke4(rgb2yuv + 87, crunch_data+64)
	poke4(rgb2yuv + 93, crunch_data+16)
	poke4(rgb2yuv + 99, crunch_data+4)
	poke4(rgb2yuv + 105, crunch_data+40)
	poke4(rgb2yuv + 111, crunch_data+16)
	poke4(rgb2yuv + 117, crunch_data+16)
	poke4(rgb2yuv + 123, crunch_data+8)
	poke4(rgb2yuv + 129, crunch_data+52)
	poke4(rgb2yuv + 135, crunch_data+16)
	poke4(rgb2yuv + 141, crunch_data+16)
	poke4(rgb2yuv + 147, crunch_data)
	poke4(rgb2yuv + 153, crunch_data+32)
	poke4(rgb2yuv + 159, crunch_data+64)
	poke4(rgb2yuv + 165, crunch_data+20)
	poke4(rgb2yuv + 171, crunch_data+4)
	poke4(rgb2yuv + 177, crunch_data+44)
	poke4(rgb2yuv + 183, crunch_data+20)
	poke4(rgb2yuv + 189, crunch_data+20)
	poke4(rgb2yuv + 195, crunch_data+8)
	poke4(rgb2yuv + 201, crunch_data+56)
	poke4(rgb2yuv + 207, crunch_data+20)
	poke4(rgb2yuv + 213, crunch_data+20)


	-- Store some floating point constants
	poke(crunch_data+24,atom_to_float32(0.257))
	poke(crunch_data+28,atom_to_float32(0.439))
	poke(crunch_data+32,atom_to_float32(-0.148))
	poke(crunch_data+36,atom_to_float32(0.504))
	poke(crunch_data+40,atom_to_float32(-0.368))
	poke(crunch_data+44,atom_to_float32(-0.291))
	poke(crunch_data+48,atom_to_float32(0.098))
	poke(crunch_data+52,atom_to_float32(-0.071))
	poke(crunch_data+56,atom_to_float32(0.439))
	poke(crunch_data+60,atom_to_float32(16.0))
	poke(crunch_data+64,atom_to_float32(128.0))
	
	crunch = allocate(640)
	if crunch<=0 then
		return 0
	end if
	poke(crunch, {
	#60,#FC,#BB,#00,#00,#00,#00,#31,#C0,#89,#03,#89,#43,#04,#89,#43,#08,#89,#C5,#8B,
	#0D,#00,#00,#00,#00,#8B,#35,#00,#00,#00,#00,#0F,#B7,#44,#35,#00,#83,#C5,#02,#89,
	#C3,#0F,#B6,#54,#35,#00,#25,#FF,#00,#00,#00,#C1,#EB,#08,#89,#15,#00,#00,#00,#00,
	#45,#89,#1D,#00,#00,#00,#00,#A3,#00,#00,#00,#00,#BB,#00,#00,#00,#00,#FF,#D3,#83,
	#FD,#03,#75,#30,#B8,#01,#00,#00,#00,#BF,#00,#00,#00,#00,#89,#07,#89,#47,#04,#89,
	#47,#08,#89,#F3,#89,#CA,#BE,#00,#00,#00,#00,#BF,#00,#00,#00,#00,#B9,#03,#00,#00,
	#00,#F3,#A5,#89,#DE,#89,#D1,#E9,#0A,#01,#00,#00,#51,#B9,#02,#00,#00,#00,#8B,#04,
	#8D,#00,#00,#00,#00,#3B,#04,#8D,#00,#00,#00,#00,#75,#50,#FF,#04,#8D,#00,#00,#00,
	#00,#8B,#04,#8D,#00,#00,#00,#00,#3D,#FF,#00,#00,#00,#75,#36,#A3,#00,#00,#00,#00,
	#B8,#00,#00,#00,#00,#A3,#00,#00,#00,#00,#8B,#04,#8D,#00,#00,#00,#00,#A3,#00,#00,
	#00,#00,#89,#0D,#00,#00,#00,#00,#BB,#00,#00,#00,#00,#FF,#D3,#31,#DB,#89,#1C,#8D,
	#00,#00,#00,#00,#E9,#A1,#00,#00,#00,#E9,#9C,#00,#00,#00,#8B,#04,#8D,#00,#00,#00,
	#00,#83,#F8,#03,#73,#4D,#21,#C0,#74,#72,#8B,#1C,#8D,#00,#00,#00,#00,#81,#FB,#00,
	#00,#00,#00,#75,#22,#89,#0D,#00,#00,#00,#00,#89,#1D,#00,#00,#00,#00,#BA,#00,#00,
	#00,#00,#FF,#D2,#31,#FF,#89,#3D,#00,#00,#00,#00,#FF,#D2,#48,#75,#E0,#EB,#41,#89,
	#0D,#00,#00,#00,#00,#89,#1D,#00,#00,#00,#00,#BA,#00,#00,#00,#00,#FF,#D2,#48,#75,
	#EA,#EB,#29,#A3,#00,#00,#00,#00,#B8,#00,#00,#00,#00,#A3,#00,#00,#00,#00,#8B,#1C,
	#8D,#00,#00,#00,#00,#89,#1D,#00,#00,#00,#00,#89,#0D,#00,#00,#00,#00,#BB,#00,#00,
	#00,#00,#FF,#D3,#B8,#01,#00,#00,#00,#89,#04,#8D,#00,#00,#00,#00,#8B,#04,#8D,#00,
	#00,#00,#00,#89,#04,#8D,#00,#00,#00,#00,#49,#0F,#89,#FD,#FE,#FF,#FF,#59,#49,#0F,
	#85,#8A,#FE,#FF,#FF,#B9,#02,#00,#00,#00,#8B,#04,#8D,#00,#00,#00,#00,#83,#F8,#03,
	#73,#4D,#21,#C0,#74,#72,#8B,#1C,#8D,#00,#00,#00,#00,#81,#FB,#00,#00,#00,#00,#75,
	#22,#89,#0D,#00,#00,#00,#00,#89,#1D,#00,#00,#00,#00,#BA,#00,#00,#00,#00,#FF,#D2,
	#31,#FF,#89,#3D,#00,#00,#00,#00,#FF,#D2,#48,#75,#E6,#EB,#41,#89,#0D,#00,#00,#00,
	#00,#89,#1D,#00,#00,#00,#00,#BA,#00,#00,#00,#00,#FF,#D2,#48,#75,#F0,#EB,#29,#A3,
	#00,#00,#00,#00,#B8,#00,#00,#00,#00,#A3,#00,#00,#00,#00,#8B,#1C,#8D,#00,#00,#00,
	#00,#89,#1D,#00,#00,#00,#00,#89,#0D,#00,#00,#00,#00,#BB,#00,#00,#00,#00,#FF,#D3,
	#49,#0F,#89,#77,#FF,#FF,#FF,#BD,#00,#00,#00,#00,#BB,#00,#00,#00,#00,#8B,#03,#8B,
	#53,#08,#8B,#5B,#04,#8B,#3D,#00,#00,#00,#00,#89,#07,#89,#5F,#04,#89,#57,#08,#89,
	#C1,#01,#D9,#01,#D1,#83,#E1,#03,#01,#CA,#89,#57,#08,#83,#C7,#0C,#BE,#00,#00,#00,
	#00,#83,#C6,#18,#89,#C1,#F3,#A4,#BE,#00,#00,#00,#00,#83,#C6,#18,#03,#76,#F8,#89,
	#D9,#F3,#A4,#BE,#00,#00,#00,#00,#83,#C6,#18,#03,#76,#FC,#89,#D1,#F3,#A4,#61,#C3})
	poke4(crunch + 3, streams)
	poke4(crunch + 21, crunch_data+96)
	poke4(crunch + 27, crunch_data+100)
	poke4(crunch + 56, crunch_data)
	poke4(crunch + 63, crunch_data+4)
	poke4(crunch + 68, crunch_data+8)
	poke4(crunch + 73, rgb2yuv)
	poke4(crunch + 90, crunch_data+80)
	poke4(crunch + 107, crunch_data+12)
	poke4(crunch + 112, crunch_data+68)
	poke4(crunch + 141, crunch_data+12)
	poke4(crunch + 148, crunch_data+68)
	poke4(crunch + 157, crunch_data+80)
	poke4(crunch + 164, crunch_data+80)
	poke4(crunch + 176, stream_data+8)
	poke4(crunch + 181, TRIGGER)
	poke4(crunch + 186, stream_data+12)
	poke4(crunch + 193, crunch_data+12)
	poke4(crunch + 198, stream_data+4)
	poke4(crunch + 204, stream_data)
	poke4(crunch + 209, stream_write_code)
	poke4(crunch + 220, crunch_data+80)
	poke4(crunch + 237, crunch_data+80)
	poke4(crunch + 253, crunch_data+68)
	poke4(crunch + 259, TRIGGER)
	poke4(crunch + 267, stream_data)
	poke4(crunch + 273, stream_data+4)
	poke4(crunch + 278, stream_write_byte)
	poke4(crunch + 288, stream_data+4)
	poke4(crunch + 301, stream_data)
	poke4(crunch + 307, stream_data+4)
	poke4(crunch + 312, stream_write_byte)
	poke4(crunch + 324, stream_data+8)
	poke4(crunch + 329, TRIGGER)
	poke4(crunch + 334, stream_data+12)
	poke4(crunch + 341, crunch_data+68)
	poke4(crunch + 347, stream_data+4)
	poke4(crunch + 353, stream_data)
	poke4(crunch + 358, stream_write_code)
	poke4(crunch + 372, crunch_data+80)
	poke4(crunch + 379, crunch_data+12)
	poke4(crunch + 386, crunch_data+68)
	poke4(crunch + 413, crunch_data+80)
	poke4(crunch + 429, crunch_data+68)
	poke4(crunch + 435, TRIGGER)
	poke4(crunch + 443, stream_data)
	poke4(crunch + 449, stream_data+4)
	poke4(crunch + 454, stream_write_byte)
	poke4(crunch + 464, stream_data+4)
	poke4(crunch + 477, stream_data)
	poke4(crunch + 483, stream_data+4)
	poke4(crunch + 488, stream_write_byte)
	poke4(crunch + 500, stream_data+8)
	poke4(crunch + 505, TRIGGER)
	poke4(crunch + 510, stream_data+12)
	poke4(crunch + 517, crunch_data+68)
	poke4(crunch + 523, stream_data+4)
	poke4(crunch + 529, stream_data)
	poke4(crunch + 534, stream_write_code)
	poke4(crunch + 548, streams)
	poke4(crunch + 553, streams)
	poke4(crunch + 567, crunch_data+104)
	poke4(crunch + 597, streams)
	poke4(crunch + 609, streams)
	poke4(crunch + 624, streams)

	return 1
end function


global procedure crunch_close_compressor()
	kill_streams()
	free(rgb2yuv)
	free(stream_write_byte)
	free(stream_write_code)
	free(crunch)
	free(crunch_data)
end procedure




		
global function crunch_encode(atom bmBits,atom dest,integer bmWidth,integer bmHeight)

	poke4(crunch_data+96,bmWidth*bmHeight)
	poke4(crunch_data+100,bmBits)
	poke4(crunch_data+104,dest)
	call(crunch)
	
	return 1
end function

	

------------------------------------------------------------------------------------------------------------

global function crunch_init_decompressor()

	decrunch_data = allocate(16)
	decrunch = allocate(365)
	if (decrunch<=0 or decrunch_data<=0) then
		return 0
	end if
	poke(decrunch, {
	#60,#FC,#8B,#35,#00,#00,#00,#00,#89,#F5,#83,#C6,#0C,#B9,#00,#00,#00,#00,#8B,#3D,
	#00,#00,#00,#00,#01,#CF,#31,#D2,#89,#15,#00,#00,#00,#00,#3B,#54,#8D,#00,#0F,#83,
	#57,#00,#00,#00,#0F,#B6,#04,#32,#42,#3D,#00,#00,#00,#00,#75,#2C,#0F,#B6,#1C,#32,
	#42,#83,#FB,#00,#75,#0D,#88,#07,#83,#C7,#03,#FF,#05,#00,#00,#00,#00,#EB,#20,#0F,
	#B6,#04,#32,#42,#88,#07,#83,#C7,#03,#FF,#05,#00,#00,#00,#00,#4B,#75,#F2,#EB,#0B,
	#88,#07,#83,#C7,#03,#FF,#05,#00,#00,#00,#00,#83,#F9,#02,#75,#0D,#A1,#00,#00,#00,
	#00,#3B,#05,#00,#00,#00,#00,#7D,#02,#EB,#9F,#83,#F9,#00,#75,#0A,#A1,#00,#00,#00,
	#00,#A3,#00,#00,#00,#00,#01,#D6,#41,#83,#F9,#03,#0F,#85,#74,#FF,#FF,#FF,#8B,#0D,
	#00,#00,#00,#00,#8D,#0C,#49,#8B,#3D,#00,#00,#00,#00,#01,#CF,#83,#F1,#FF,#41,#89,
	#CE,#0F,#B7,#04,#3E,#83,#C6,#02,#89,#C3,#0F,#B6,#2C,#3E,#25,#FF,#00,#00,#00,#C1,
	#EB,#08,#83,#E8,#10,#BE,#9F,#12,#00,#00,#F7,#E6,#81,#EB,#80,#00,#00,#00,#81,#ED,
	#80,#00,#00,#00,#89,#C6,#B8,#89,#19,#00,#00,#F7,#ED,#01,#F0,#C1,#E8,#0C,#83,#F8,
	#00,#7D,#02,#31,#C0,#3D,#FF,#00,#00,#00,#7E,#05,#B8,#FF,#00,#00,#00,#88,#04,#39,
	#41,#B8,#41,#06,#00,#00,#F7,#EB,#83,#F0,#FF,#40,#01,#F0,#A3,#00,#00,#00,#00,#B8,
	#02,#0D,#00,#00,#F7,#ED,#83,#F0,#FF,#40,#03,#05,#00,#00,#00,#00,#C1,#F8,#0C,#83,
	#F8,#00,#7D,#02,#31,#C0,#3D,#FF,#00,#00,#00,#7E,#05,#B8,#FF,#00,#00,#00,#88,#04,
	#39,#41,#B8,#4A,#20,#00,#00,#F7,#EB,#01,#F0,#C1,#F8,#0C,#83,#F8,#00,#7D,#02,#31,
	#C0,#3D,#FF,#00,#00,#00,#7E,#05,#B8,#FF,#00,#00,#00,#88,#04,#39,#41,#0F,#85,#48,
	#FF,#FF,#FF,#61,#C3})
	poke4(decrunch + 4, decrunch_data)
	poke4(decrunch + 20, decrunch_data+4)
	poke4(decrunch + 30, decrunch_data+8)
	poke4(decrunch + 50, TRIGGER)
	poke4(decrunch + 73, decrunch_data+8)
	poke4(decrunch + 91, decrunch_data+8)
	poke4(decrunch + 107, decrunch_data+8)
	poke4(decrunch + 117, decrunch_data+8)
	poke4(decrunch + 123, decrunch_data+12)
	poke4(decrunch + 137, decrunch_data+8)
	poke4(decrunch + 142, decrunch_data+12)
	poke4(decrunch + 160, decrunch_data+8)
	poke4(decrunch + 169, decrunch_data+4)
	poke4(decrunch + 275, decrunch_data+12)
	poke4(decrunch + 292, decrunch_data+12)

	return 1
end function


global procedure crunch_close_decompressor()
	free(decrunch)
	free(decrunch_data)
end procedure


global function crunch_decode(atom compressed,atom dest)
	poke4(decrunch_data,compressed)
	poke4(decrunch_data+4,dest)
	call(decrunch)
	
	return 1
end function

	
	

