-- Crunch bitmap codec (lossy version)
-- /Mic, 2002
--
-- Takes a 24-bit RGB bitmap as input, transforms it to YUV 4:1:1 (by averaging U & V for each 2x2 block of pixels),
-- and then applies the crunch algorithm on the result.
--


atom lcrunch_streams,lcrunch_stream_data
atom lcrunch,lcrunch_data,lcrunch_stream_write_byte,lcrunch_stream_write_word,lcrunch_stream_write_code
atom lcrunch_rgb2yuv,lcrunch_rgb2yuv_tb,yuv411buffer,rgb2yuv411,crunch411
atom delcrunch,delcrunch_data,yuv411buffer2
integer delcrunch_bmWidth



function lcrunch_init_streams(integer bmSize)
	lcrunch_streams = allocate(24+bmSize+16)
	if lcrunch_streams<=0 then 
		return 0
	end if
	lcrunch_stream_data = lcrunch_streams+24+bmSize
	return 1
end function



procedure lcrunch_reset_streams()
	poke4(lcrunch_streams,{0,0,0})
end procedure



procedure lcrunch_kill_streams()
	free(lcrunch_streams)
end procedure




constant TRIGGER2 = #FE



global function lcrunch_init_compressor(integer bmWidth,integer bmHeight)
	if not lcrunch_init_streams(bmWidth*bmHeight) then
		return 0
	end if
	lcrunch_reset_streams()

	yuv411buffer = allocate(floor(bmWidth*bmHeight*3/2))
	if yuv411buffer<=0 then
		return 0
	end if
	
	lcrunch_data = allocate(128)
	if lcrunch_data<=0 then
		return 0
	end if

	lcrunch_rgb2yuv_tb = allocate(6912)
	if lcrunch_rgb2yuv_tb<=0 then
		return 0
	end if
	-- Set up RGB->YUV conversion table
	for i=0 to 255 do
		poke(lcrunch_rgb2yuv_tb+i,floor(0.257*i))
		poke(lcrunch_rgb2yuv_tb+i+256,floor(0.504*i))
		poke(lcrunch_rgb2yuv_tb+i+512,floor(0.098*i))
		poke(lcrunch_rgb2yuv_tb+(i*4)+768,int_to_bytes(floor(0.439*i)))
		poke(lcrunch_rgb2yuv_tb+(i*4)+768+1024,int_to_bytes(floor(-0.368*i)))
		poke(lcrunch_rgb2yuv_tb+(i*4)+768+2048,int_to_bytes(floor(-0.071*i)))
		poke(lcrunch_rgb2yuv_tb+(i*4)+768+3072,int_to_bytes(floor(-0.148*i)))
		poke(lcrunch_rgb2yuv_tb+(i*4)+768+4096,int_to_bytes(floor(-0.291*i)))
		poke(lcrunch_rgb2yuv_tb+(i*4)+768+5120,int_to_bytes(floor(0.439*i)))
	end for
	
	
	-- Write a byte (8 bits) to one of the streams
	lcrunch_stream_write_byte = allocate(39)
	poke(lcrunch_stream_write_byte, {
	#60,#8B,#0D,#00,#00,#00,#00,#B8,#00,#00,#00,#00,#8B,#14,#88,#FF,#04,#88,#83,#C0,
	#0C,#8B,#1C,#88,#A1,#00,#00,#00,#00,#BD,#00,#00,#00,#00,#88,#04,#1A,#61,#C3})
	poke4(lcrunch_stream_write_byte + 3, lcrunch_stream_data)
	poke4(lcrunch_stream_write_byte + 8, lcrunch_streams)
	poke4(lcrunch_stream_write_byte + 25, lcrunch_stream_data+4)


	-- Write a word (16 bits) to one of the streams
	lcrunch_stream_write_word = allocate(40)
	poke(lcrunch_stream_write_word, {
	#60,#8B,#0D,#00,#00,#00,#00,#B8,#00,#00,#00,#00,#8B,#14,#88,#89,#D5,#83,#C5,#02,
	#89,#2C,#88,#83,#C0,#0C,#8B,#1C,#88,#A1,#00,#00,#00,#00,#66,#89,#04,#1A,#61,#C3})
	poke4(lcrunch_stream_write_word + 3, lcrunch_stream_data)
	poke4(lcrunch_stream_write_word + 8, lcrunch_streams)
	poke4(lcrunch_stream_write_word + 30, lcrunch_stream_data+4)


	lcrunch_stream_write_code = allocate(47)
	poke(lcrunch_stream_write_code, {
	#60,#BB,#00,#00,#00,#00,#8B,#2D,#00,#00,#00,#00,#A1,#00,#00,#00,#00,#C1,#E0,#08,
	#A0,#00,#00,#00,#00,#A3,#00,#00,#00,#00,#FF,#D3,#BB,#00,#00,#00,#00,#89,#2D,#00,
	#00,#00,#00,#FF,#D3,#61,#C3})
	poke4(lcrunch_stream_write_code + 2, lcrunch_stream_write_word)
	poke4(lcrunch_stream_write_code + 8, lcrunch_stream_data+4)
	poke4(lcrunch_stream_write_code + 13, lcrunch_stream_data+8)
	poke4(lcrunch_stream_write_code + 21, lcrunch_stream_data+12)
	poke4(lcrunch_stream_write_code + 26, lcrunch_stream_data+4)
	poke4(lcrunch_stream_write_code + 33, lcrunch_stream_write_byte)
	poke4(lcrunch_stream_write_code + 39, lcrunch_stream_data+4)


	-- Table based RGB->YUV conversion
	lcrunch_rgb2yuv = allocate(110)
	poke(lcrunch_rgb2yuv, {
	#60,#8B,#35,#00,#00,#00,#00,#0F,#B7,#06,#89,#C3,#0F,#B6,#4E,#02,#C1,#EB,#08,#25,
	#FF,#00,#00,#00,#0F,#B6,#91,#00,#00,#00,#00,#02,#93,#00,#00,#00,#00,#02,#90,#00,
	#00,#00,#00,#83,#C2,#10,#89,#17,#8B,#14,#8D,#00,#00,#00,#00,#03,#14,#9D,#00,#00,
	#00,#00,#03,#14,#85,#00,#00,#00,#00,#81,#C2,#80,#00,#00,#00,#89,#57,#04,#8B,#14,
	#8D,#00,#00,#00,#00,#03,#14,#9D,#00,#00,#00,#00,#03,#14,#85,#00,#00,#00,#00,#81,
	#C2,#80,#00,#00,#00,#89,#57,#08,#61,#C3})
	poke4(lcrunch_rgb2yuv + 3, lcrunch_data+16)
	poke4(lcrunch_rgb2yuv + 27, lcrunch_rgb2yuv_tb)
	poke4(lcrunch_rgb2yuv + 33, lcrunch_rgb2yuv_tb+256)
	poke4(lcrunch_rgb2yuv + 39, lcrunch_rgb2yuv_tb+512)
	poke4(lcrunch_rgb2yuv + 51, lcrunch_rgb2yuv_tb+768)
	poke4(lcrunch_rgb2yuv + 58, lcrunch_rgb2yuv_tb+768+1024)
	poke4(lcrunch_rgb2yuv + 65, lcrunch_rgb2yuv_tb+768+2048)
	poke4(lcrunch_rgb2yuv + 81, lcrunch_rgb2yuv_tb+768+3072)
	poke4(lcrunch_rgb2yuv + 88, lcrunch_rgb2yuv_tb+768+4096)
	poke4(lcrunch_rgb2yuv + 95, lcrunch_rgb2yuv_tb+768+5120)


	-- Subsample the U & V components to 1 each per four Y's. Automatically gives a 2:1 compression ratio, but
	-- degrades the picture quality somewhat.
	rgb2yuv411 = allocate(236)
	poke(rgb2yuv411, {
	#60,#FC,#8B,#35,#00,#00,#00,#00,#BF,#00,#00,#00,#00,#BA,#00,#00,#00,#00,#BD,#00,
	#00,#00,#00,#57,#89,#35,#00,#00,#00,#00,#BF,#00,#00,#00,#00,#BB,#00,#00,#00,#00,
	#FF,#D3,#8D,#46,#03,#A3,#00,#00,#00,#00,#BF,#00,#00,#00,#00,#FF,#D3,#8B,#0D,#00,
	#00,#00,#00,#8D,#0C,#49,#01,#C8,#A3,#00,#00,#00,#00,#BF,#00,#00,#00,#00,#FF,#D3,
	#83,#E8,#03,#A3,#00,#00,#00,#00,#BF,#00,#00,#00,#00,#FF,#D3,#5F,#A1,#00,#00,#00,
	#00,#8B,#1D,#00,#00,#00,#00,#8B,#0D,#00,#00,#00,#00,#88,#07,#88,#5F,#01,#A1,#00,
	#00,#00,#00,#88,#4F,#02,#88,#47,#03,#83,#C7,#04,#A1,#04,#00,#00,#00,#03,#05,#04,
	#00,#00,#00,#03,#05,#04,#00,#00,#00,#03,#05,#04,#00,#00,#00,#C1,#E8,#02,#88,#07,
	#47,#A1,#08,#00,#00,#00,#03,#05,#08,#00,#00,#00,#03,#05,#08,#00,#00,#00,#03,#05,
	#08,#00,#00,#00,#C1,#E8,#02,#88,#07,#47,#83,#C6,#06,#83,#C5,#02,#3B,#2D,#00,#00,
	#00,#00,#0F,#8C,#47,#FF,#FF,#FF,#8B,#0D,#00,#00,#00,#00,#8D,#0C,#49,#01,#CE,#83,
	#C2,#02,#3B,#15,#00,#00,#00,#00,#0F,#8C,#28,#FF,#FF,#FF,#61,#C3})
	poke4(rgb2yuv411 + 4, lcrunch_data)
	poke4(rgb2yuv411 + 9, yuv411buffer)
	poke4(rgb2yuv411 + 26, lcrunch_data+16)
	poke4(rgb2yuv411 + 31, lcrunch_data+20)
	poke4(rgb2yuv411 + 36, lcrunch_rgb2yuv)
	poke4(rgb2yuv411 + 46, lcrunch_data+16)
	poke4(rgb2yuv411 + 51, lcrunch_data+32)
	poke4(rgb2yuv411 + 59, lcrunch_data+4)
	poke4(rgb2yuv411 + 69, lcrunch_data+16)
	poke4(rgb2yuv411 + 74, lcrunch_data+56)
	poke4(rgb2yuv411 + 84, lcrunch_data+16)
	poke4(rgb2yuv411 + 89, lcrunch_data+44)
	poke4(rgb2yuv411 + 97, lcrunch_data+20)
	poke4(rgb2yuv411 + 103, lcrunch_data+32)
	poke4(rgb2yuv411 + 109, lcrunch_data+44)
	poke4(rgb2yuv411 + 119, lcrunch_data+56)
	poke4(rgb2yuv411 + 133, lcrunch_data + 20 + peek4s(rgb2yuv411 + 133))
	poke4(rgb2yuv411 + 139, lcrunch_data + 32 + peek4s(rgb2yuv411 + 139))
	poke4(rgb2yuv411 + 145, lcrunch_data + 44 + peek4s(rgb2yuv411 + 145))
	poke4(rgb2yuv411 + 151, lcrunch_data + 56 + peek4s(rgb2yuv411 + 151))
	poke4(rgb2yuv411 + 162, lcrunch_data + 20 + peek4s(rgb2yuv411 + 162))
	poke4(rgb2yuv411 + 168, lcrunch_data + 32 + peek4s(rgb2yuv411 + 168))
	poke4(rgb2yuv411 + 174, lcrunch_data + 44 + peek4s(rgb2yuv411 + 174))
	poke4(rgb2yuv411 + 180, lcrunch_data + 56 + peek4s(rgb2yuv411 + 180))
	poke4(rgb2yuv411 + 198, lcrunch_data+4)
	poke4(rgb2yuv411 + 210, lcrunch_data+4)
	poke4(rgb2yuv411 + 224, lcrunch_data+8)
	
	
	crunch411 = allocate(468)
	poke(crunch411, {
	#60,#BE,#00,#00,#00,#00,#8B,#0D,#00,#00,#00,#00,#BD,#00,#00,#00,#00,#B8,#FF,#FF,
	#FF,#FF,#A3,#00,#00,#00,#00,#A3,#04,#00,#00,#00,#A3,#08,#00,#00,#00,#BA,#00,#00,
	#00,#00,#83,#FD,#04,#72,#07,#42,#83,#FD,#05,#75,#01,#42,#8B,#04,#95,#00,#00,#00,
	#00,#83,#F8,#FF,#75,#1B,#0F,#B6,#06,#89,#04,#95,#00,#00,#00,#00,#B8,#01,#00,#00,
	#00,#89,#04,#95,#00,#00,#00,#00,#E9,#E7,#00,#00,#00,#0F,#B6,#06,#3B,#04,#95,#00,
	#00,#00,#00,#75,#4B,#FF,#04,#95,#00,#00,#00,#00,#8B,#1C,#95,#00,#00,#00,#00,#81,
	#FB,#FF,#00,#00,#00,#0F,#85,#C1,#00,#00,#00,#89,#15,#00,#00,#00,#00,#89,#1D,#00,
	#00,#00,#00,#BB,#00,#00,#00,#00,#89,#1D,#00,#00,#00,#00,#A3,#00,#00,#00,#00,#BB,
	#00,#00,#00,#00,#FF,#D3,#31,#DB,#89,#1C,#95,#00,#00,#00,#00,#E9,#90,#00,#00,#00,
	#8B,#1C,#95,#00,#00,#00,#00,#21,#DB,#74,#6F,#83,#FB,#03,#73,#41,#8B,#04,#95,#00,
	#00,#00,#00,#3D,#00,#00,#00,#00,#75,#1C,#89,#15,#00,#00,#00,#00,#B8,#00,#00,#00,
	#00,#A3,#00,#00,#00,#00,#B8,#00,#00,#00,#00,#FF,#D0,#4B,#75,#EC,#EB,#40,#89,#15,
	#00,#00,#00,#00,#A3,#00,#00,#00,#00,#BF,#00,#00,#00,#00,#FF,#D7,#4B,#75,#F1,#EB,
	#29,#B8,#00,#00,#00,#00,#A3,#00,#00,#00,#00,#89,#1D,#00,#00,#00,#00,#89,#15,#00,
	#00,#00,#00,#8B,#04,#95,#00,#00,#00,#00,#BB,#00,#00,#00,#00,#A3,#00,#00,#00,#00,
	#FF,#D3,#0F,#B6,#06,#89,#04,#95,#00,#00,#00,#00,#B8,#01,#00,#00,#00,#89,#04,#95,
	#00,#00,#00,#00,#46,#45,#83,#FD,#06,#75,#02,#31,#ED,#49,#0F,#85,#D1,#FE,#FF,#FF,
	#BA,#02,#00,#00,#00,#8B,#1C,#95,#00,#00,#00,#00,#83,#FB,#03,#7D,#41,#8B,#04,#95,
	#00,#00,#00,#00,#3D,#00,#00,#00,#00,#75,#1C,#89,#15,#00,#00,#00,#00,#B8,#00,#00,
	#00,#00,#A3,#00,#00,#00,#00,#B8,#00,#00,#00,#00,#FF,#D0,#4B,#75,#EC,#EB,#40,#89,
	#15,#00,#00,#00,#00,#A3,#00,#00,#00,#00,#BF,#00,#00,#00,#00,#FF,#D7,#4B,#75,#F1,
	#EB,#29,#B8,#00,#00,#00,#00,#A3,#00,#00,#00,#00,#89,#1D,#00,#00,#00,#00,#89,#15,
	#00,#00,#00,#00,#8B,#04,#95,#00,#00,#00,#00,#BB,#00,#00,#00,#00,#A3,#00,#00,#00,
	#00,#FF,#D3,#4A,#79,#87,#61,#C3})
	poke4(crunch411 + 2, yuv411buffer)
	poke4(crunch411 + 8, lcrunch_data+68)
	poke4(crunch411 + 23, lcrunch_data+20)
	poke4(crunch411 + 28, lcrunch_data+20 + peek4s(crunch411 + 28))
	poke4(crunch411 + 33, lcrunch_data+20 + peek4s(crunch411 + 33))
	poke4(crunch411 + 57, lcrunch_data+20)
	poke4(crunch411 + 72, lcrunch_data+20)
	poke4(crunch411 + 84, lcrunch_data+32)
	poke4(crunch411 + 99, lcrunch_data+20)
	poke4(crunch411 + 108, lcrunch_data+32)
	poke4(crunch411 + 115, lcrunch_data+32)
	poke4(crunch411 + 133, lcrunch_stream_data)
	poke4(crunch411 + 139, lcrunch_stream_data+8)
	poke4(crunch411 + 144, TRIGGER2)
	poke4(crunch411 + 150, lcrunch_stream_data+12)
	poke4(crunch411 + 155, lcrunch_stream_data+4)
	poke4(crunch411 + 160, lcrunch_stream_write_code)
	poke4(crunch411 + 171, lcrunch_data+32)
	poke4(crunch411 + 183, lcrunch_data+32)
	poke4(crunch411 + 199, lcrunch_data+20)
	poke4(crunch411 + 204, TRIGGER2)
	poke4(crunch411 + 212, lcrunch_stream_data)
	poke4(crunch411 + 217, TRIGGER2)
	poke4(crunch411 + 222, lcrunch_stream_data+4)
	poke4(crunch411 + 227, lcrunch_stream_write_word)
	poke4(crunch411 + 240, lcrunch_stream_data)
	poke4(crunch411 + 245, lcrunch_stream_data+4)
	poke4(crunch411 + 250, lcrunch_stream_write_byte)
	poke4(crunch411 + 262, TRIGGER2)
	poke4(crunch411 + 267, lcrunch_stream_data+12)
	poke4(crunch411 + 273, lcrunch_stream_data+8)
	poke4(crunch411 + 279, lcrunch_stream_data)
	poke4(crunch411 + 286, lcrunch_data+20)
	poke4(crunch411 + 291, lcrunch_stream_write_code)
	poke4(crunch411 + 296, lcrunch_stream_data+4)
	poke4(crunch411 + 308, lcrunch_data+20)
	poke4(crunch411 + 320, lcrunch_data+32)
	poke4(crunch411 + 348, lcrunch_data+32)
	poke4(crunch411 + 360, lcrunch_data+20)
	poke4(crunch411 + 365, TRIGGER2)
	poke4(crunch411 + 373, lcrunch_stream_data)
	poke4(crunch411 + 378, TRIGGER2)
	poke4(crunch411 + 383, lcrunch_stream_data+4)
	poke4(crunch411 + 388, lcrunch_stream_write_word)
	poke4(crunch411 + 401, lcrunch_stream_data)
	poke4(crunch411 + 406, lcrunch_stream_data+4)
	poke4(crunch411 + 411, lcrunch_stream_write_byte)
	poke4(crunch411 + 423, TRIGGER2)
	poke4(crunch411 + 428, lcrunch_stream_data+12)
	poke4(crunch411 + 434, lcrunch_stream_data+8)
	poke4(crunch411 + 440, lcrunch_stream_data)
	poke4(crunch411 + 447, lcrunch_data+20)
	poke4(crunch411 + 452, lcrunch_stream_write_code)
	poke4(crunch411 + 457, lcrunch_stream_data+4)
	
	
	lcrunch = allocate(77)
	poke(lcrunch, {
	#60,#BB,#00,#00,#00,#00,#FF,#D3,#BB,#00,#00,#00,#00,#FF,#D3,#BE,#00,#00,#00,#00,
	#89,#F5,#8B,#3D,#00,#00,#00,#00,#B9,#03,#00,#00,#00,#F3,#A5,#8B,#75,#10,#8B,#4D,
	#04,#03,#7D,#00,#F3,#A4,#8B,#75,#14,#8B,#4D,#08,#F3,#A4,#8B,#45,#00,#03,#45,#04,
	#03,#45,#08,#83,#E0,#03,#8B,#3D,#00,#00,#00,#00,#01,#47,#08,#61,#C3})
	poke4(lcrunch + 2, rgb2yuv411)
	poke4(lcrunch + 9, crunch411)
	poke4(lcrunch + 16, lcrunch_streams)
	poke4(lcrunch + 24, lcrunch_data+12)
	poke4(lcrunch + 68, lcrunch_data+12)

	return 1
end function



global procedure lcrunch_close_compressor()
	free(lcrunch)
	free(lcrunch_rgb2yuv)
	free(lcrunch_rgb2yuv_tb)
	free(rgb2yuv411)
	free(crunch411)
	free(yuv411buffer)
	lcrunch_kill_streams()
end procedure



global function lcrunch_encode(atom bmBits,atom dest,integer bmWidth,integer bmHeight)

	poke4(lcrunch_streams,{0,0,0,dest+12,lcrunch_streams+24,lcrunch_streams+24+floor(bmWidth*bmHeight/2)})
	poke4(lcrunch_data,{bmBits,bmWidth,bmHeight,dest})
	poke4(lcrunch_data+68,floor(bmWidth*bmHeight*3/2))	
	call(lcrunch)
	
	return 1
end function




global function lcrunch_init_decompressor(integer bmWidth,integer bmHeight)

	yuv411buffer2 = allocate(floor(bmWidth*bmHeight*3/2))
	delcrunch_data = allocate(64)
	delcrunch = allocate(1060)

	if (yuv411buffer2<=0 or delcrunch_data<=0 or delcrunch<=0) then
		return 0
	end if
	
	delcrunch_bmWidth = bmWidth
	
	poke(delcrunch, {
	#60,#FC,#8B,#35,#00,#00,#00,#00,#89,#35,#00,#00,#00,#00,#83,#C6,#0C,#B9,#00,#00,
	#00,#00,#BF,#00,#00,#00,#00,#83,#F9,#01,#75,#03,#83,#C7,#04,#83,#F9,#02,#75,#03,
	#83,#C7,#05,#BD,#00,#00,#00,#00,#31,#D2,#89,#15,#00,#00,#00,#00,#A1,#00,#00,#00,
	#00,#3B,#14,#88,#0F,#83,#DC,#00,#00,#00,#0F,#B6,#04,#32,#42,#3D,#00,#00,#00,#00,
	#0F,#85,#81,#00,#00,#00,#0F,#B6,#1C,#32,#42,#21,#DB,#75,#36,#88,#07,#31,#C0,#BB,
	#01,#00,#00,#00,#21,#C9,#0F,#44,#C3,#BB,#06,#00,#00,#00,#01,#C5,#21,#C9,#0F,#45,
	#C3,#BB,#02,#00,#00,#00,#01,#C7,#31,#C0,#83,#FD,#04,#0F,#44,#E8,#0F,#44,#C3,#01,
	#C7,#FF,#05,#00,#00,#00,#00,#EB,#76,#52,#0F,#B6,#14,#32,#01,#1D,#00,#00,#00,#00,
	#88,#17,#53,#31,#C0,#BB,#01,#00,#00,#00,#21,#C9,#0F,#44,#C3,#BB,#06,#00,#00,#00,
	#01,#C5,#21,#C9,#0F,#45,#C3,#BB,#02,#00,#00,#00,#01,#C7,#31,#C0,#83,#FD,#04,#0F,
	#44,#E8,#0F,#44,#C3,#01,#C7,#5B,#4B,#75,#CD,#5A,#42,#EB,#34,#88,#07,#31,#C0,#BB,
	#01,#00,#00,#00,#21,#C9,#0F,#44,#C3,#BB,#06,#00,#00,#00,#01,#C5,#21,#C9,#0F,#45,
	#C3,#BB,#02,#00,#00,#00,#01,#C7,#31,#C0,#83,#FD,#04,#0F,#44,#E8,#0F,#44,#C3,#01,
	#C7,#FF,#05,#00,#00,#00,#00,#83,#F9,#02,#75,#0D,#A1,#00,#00,#00,#00,#3B,#05,#00,
	#00,#00,#00,#7D,#05,#E9,#16,#FF,#FF,#FF,#83,#F9,#01,#75,#0A,#A1,#00,#00,#00,#00,
	#A3,#00,#00,#00,#00,#01,#D6,#41,#83,#F9,#03,#0F,#85,#D9,#FE,#FF,#FF,#A1,#00,#00,
	#00,#00,#8D,#04,#40,#A3,#00,#00,#00,#00,#31,#C0,#A3,#00,#00,#00,#00,#BE,#00,#00,
	#00,#00,#8B,#3D,#00,#00,#00,#00,#0F,#B6,#06,#0F,#B7,#5E,#04,#89,#D9,#81,#E3,#FF,
	#00,#00,#00,#C1,#E9,#08,#81,#EB,#80,#00,#00,#00,#81,#E9,#80,#00,#00,#00,#56,#83,
	#E8,#10,#BE,#9F,#12,#00,#00,#F7,#EE,#A3,#00,#00,#00,#00,#B8,#89,#19,#00,#00,#F7,
	#E9,#03,#05,#00,#00,#00,#00,#31,#F6,#C1,#F8,#0C,#83,#F8,#00,#0F,#4C,#C6,#BE,#FF,
	#00,#00,#00,#3D,#FF,#00,#00,#00,#0F,#4F,#C6,#88,#07,#B8,#41,#06,#00,#00,#F7,#EB,
	#8B,#35,#00,#00,#00,#00,#29,#C6,#B8,#02,#0D,#00,#00,#F7,#E9,#29,#C6,#89,#F0,#C1,
	#F8,#0C,#31,#F6,#83,#F8,#00,#0F,#4C,#C6,#BE,#FF,#00,#00,#00,#3D,#FF,#00,#00,#00,
	#0F,#4F,#C6,#88,#47,#01,#B8,#4A,#20,#00,#00,#F7,#EB,#03,#05,#00,#00,#00,#00,#C1,
	#F8,#0C,#31,#F6,#83,#F8,#00,#0F,#4C,#C6,#BE,#FF,#00,#00,#00,#3D,#FF,#00,#00,#00,
	#0F,#4F,#C6,#88,#47,#02,#5E,#83,#C7,#03,#0F,#B6,#46,#01,#56,#83,#E8,#10,#BE,#9F,
	#12,#00,#00,#F7,#EE,#A3,#00,#00,#00,#00,#B8,#89,#19,#00,#00,#F7,#E9,#03,#05,#00,
	#00,#00,#00,#31,#F6,#C1,#F8,#0C,#83,#F8,#00,#0F,#4C,#C6,#BE,#FF,#00,#00,#00,#3D,
	#FF,#00,#00,#00,#0F,#4F,#C6,#88,#07,#B8,#41,#06,#00,#00,#F7,#EB,#8B,#35,#00,#00,
	#00,#00,#29,#C6,#B8,#02,#0D,#00,#00,#F7,#E9,#29,#C6,#89,#F0,#C1,#F8,#0C,#31,#F6,
	#83,#F8,#00,#0F,#4C,#C6,#BE,#FF,#00,#00,#00,#3D,#FF,#00,#00,#00,#0F,#4F,#C6,#88,
	#47,#01,#B8,#4A,#20,#00,#00,#F7,#EB,#03,#05,#00,#00,#00,#00,#C1,#F8,#0C,#31,#F6,
	#83,#F8,#00,#0F,#4C,#C6,#BE,#FF,#00,#00,#00,#3D,#FF,#00,#00,#00,#0F,#4F,#C6,#88,
	#47,#02,#5E,#57,#83,#EF,#03,#0F,#B6,#46,#02,#03,#3D,#00,#00,#00,#00,#56,#83,#E8,
	#10,#BE,#9F,#12,#00,#00,#F7,#EE,#A3,#00,#00,#00,#00,#B8,#89,#19,#00,#00,#F7,#E9,
	#03,#05,#00,#00,#00,#00,#31,#F6,#C1,#F8,#0C,#83,#F8,#00,#0F,#4C,#C6,#BE,#FF,#00,
	#00,#00,#3D,#FF,#00,#00,#00,#0F,#4F,#C6,#88,#07,#B8,#41,#06,#00,#00,#F7,#EB,#8B,
	#35,#00,#00,#00,#00,#29,#C6,#B8,#02,#0D,#00,#00,#F7,#E9,#29,#C6,#89,#F0,#C1,#F8,
	#0C,#31,#F6,#83,#F8,#00,#0F,#4C,#C6,#BE,#FF,#00,#00,#00,#3D,#FF,#00,#00,#00,#0F,
	#4F,#C6,#88,#47,#01,#B8,#4A,#20,#00,#00,#F7,#EB,#03,#05,#00,#00,#00,#00,#C1,#F8,
	#0C,#31,#F6,#83,#F8,#00,#0F,#4C,#C6,#BE,#FF,#00,#00,#00,#3D,#FF,#00,#00,#00,#0F,
	#4F,#C6,#88,#47,#02,#5E,#83,#C7,#03,#0F,#B6,#46,#03,#56,#83,#E8,#10,#BE,#9F,#12,
	#00,#00,#F7,#EE,#A3,#00,#00,#00,#00,#B8,#89,#19,#00,#00,#F7,#E9,#03,#05,#00,#00,
	#00,#00,#31,#F6,#C1,#F8,#0C,#83,#F8,#00,#0F,#4C,#C6,#BE,#FF,#00,#00,#00,#3D,#FF,
	#00,#00,#00,#0F,#4F,#C6,#88,#07,#B8,#41,#06,#00,#00,#F7,#EB,#8B,#35,#00,#00,#00,
	#00,#29,#C6,#B8,#02,#0D,#00,#00,#F7,#E9,#29,#C6,#89,#F0,#C1,#F8,#0C,#31,#F6,#83,
	#F8,#00,#0F,#4C,#C6,#BE,#FF,#00,#00,#00,#3D,#FF,#00,#00,#00,#0F,#4F,#C6,#88,#47,
	#01,#B8,#4A,#20,#00,#00,#F7,#EB,#03,#05,#00,#00,#00,#00,#C1,#F8,#0C,#31,#F6,#83,
	#F8,#00,#0F,#4C,#C6,#BE,#FF,#00,#00,#00,#3D,#FF,#00,#00,#00,#0F,#4F,#C6,#88,#47,
	#02,#5E,#5F,#83,#C6,#06,#83,#C7,#03,#B8,#02,#00,#00,#00,#01,#05,#00,#00,#00,#00,
	#A1,#00,#00,#00,#00,#3B,#05,#00,#00,#00,#00,#75,#0D,#03,#3D,#00,#00,#00,#00,#31,
	#C0,#A3,#00,#00,#00,#00,#FF,#0D,#00,#00,#00,#00,#0F,#85,#3A,#FD,#FF,#FF,#61,#C3})
	poke4(delcrunch + 4, delcrunch_data)
	poke4(delcrunch + 10, delcrunch_data+28)
	poke4(delcrunch + 23, yuv411buffer2)
	poke4(delcrunch + 52, delcrunch_data+20)
	poke4(delcrunch + 57, delcrunch_data+28)
	poke4(delcrunch + 76, TRIGGER2)
	poke4(delcrunch + 143, delcrunch_data+20)
	poke4(delcrunch + 156, delcrunch_data+20)
	poke4(delcrunch + 263, delcrunch_data+20)
	poke4(delcrunch + 273, delcrunch_data+20)
	poke4(delcrunch + 279, delcrunch_data+24)
	poke4(delcrunch + 296, delcrunch_data+20)
	poke4(delcrunch + 301, delcrunch_data+24)
	poke4(delcrunch + 318, delcrunch_data+8)
	poke4(delcrunch + 326, delcrunch_data+12)
	poke4(delcrunch + 333, delcrunch_data+16)
	poke4(delcrunch + 338, yuv411buffer2)
	poke4(delcrunch + 344, delcrunch_data+4)
	poke4(delcrunch + 390, delcrunch_data+28)
	poke4(delcrunch + 403, delcrunch_data+28)
	poke4(delcrunch + 442, delcrunch_data+28)
	poke4(delcrunch + 495, delcrunch_data+28)
	poke4(delcrunch + 546, delcrunch_data+28)
	poke4(delcrunch + 559, delcrunch_data+28)
	poke4(delcrunch + 598, delcrunch_data+28)
	poke4(delcrunch + 651, delcrunch_data+28)
	poke4(delcrunch + 693, delcrunch_data+12)
	poke4(delcrunch + 709, delcrunch_data+28)
	poke4(delcrunch + 722, delcrunch_data+28)
	poke4(delcrunch + 761, delcrunch_data+28)
	poke4(delcrunch + 814, delcrunch_data+28)
	poke4(delcrunch + 865, delcrunch_data+28)
	poke4(delcrunch + 878, delcrunch_data+28)
	poke4(delcrunch + 917, delcrunch_data+28)
	poke4(delcrunch + 970, delcrunch_data+28)
	poke4(delcrunch + 1016, delcrunch_data+16)
	poke4(delcrunch + 1021, delcrunch_data+8)
	poke4(delcrunch + 1027, delcrunch_data+16)
	poke4(delcrunch + 1035, delcrunch_data+12)
	poke4(delcrunch + 1042, delcrunch_data+16)
	poke4(delcrunch + 1048, delcrunch_data+20)
	
	return 1
end function



global procedure lcrunch_close_decompressor()
	free(yuv411buffer2)
	free(delcrunch)
	free(delcrunch_data)
end procedure



global function lcrunch_decode(atom compressed,atom dest)
	poke4(delcrunch_data,{compressed,dest,delcrunch_bmWidth})
	call(delcrunch)
	return 1
end function




