-- Test program for AVIRAW
-- /Mic, 2002

include aviraw.e
include graphics.e				-- Needed for get_position()


constant NUMFRAMES=100				-- Write 100 frames (4s @ 25fps)


sequence pos
atom ctime,fps,lastKnownFps
atom totsize
lastKnownFps = 100
totsize = 0



-- Allocate memory for a single frame
atom frame
frame = allocate(128*100*3)
if (frame<=0) then
	abort(0)
end if


procedure draw_frame(integer boxpos)
	-- Clear the frame
	mem_set(frame,0,128*100*3)

	-- Draw a white box
	for i=0 to 11 do
		for j=0 to 8 do
			poke(frame+((46+i)*128*3)+(boxpos*3)+(j*4),{#FF,#FF,#FF,#FF})
		end for
	end for
end procedure

		

-- ******** The actual AVI creation code ********

AVI_SetFileName("demo.avi")			-- Set the output file name. 
AVI_SetFrameRate(25)				-- Set the framerate to 25 fps
AVI_SetFrameFormat(128,100,24)			-- Set the output format to 128x100, 24-bit color
AVI_SetCompressor(AVI_LCRUNCH)			-- Use the CrunchYUV compressor

if not AVI_Begin() then				-- Initialize the AVI output engine
	puts(1,AVI_GetLastErrorMessage())
	abort(0)
end if

puts(1,"Created file: demo.avi\n")
puts(1,"Writing stream:")


for i=1 to NUMFRAMES do	
	draw_frame(i)
	if not AVI_AddFrame(frame) then		-- Add another frame
		-- An error occured
	end if

	ctime = AVI_GetLastCompressionTime()
	if ctime=0 then
		fps = lastKnownFps
	else
		fps = 1/ctime
		lastKnownFps = fps
	end if
	totsize += AVI_GetLastFrameSize()
	
	pos = get_position()
	position(pos[1],17)
	printf(1,"%d%% | %1.1f fps | %d kbps",{100*i/NUMFRAMES,fps,AVI_GetLastFrameSize()*25/128})
end for
printf(1,"\nDone!\nAvg. compression ratio = %1.2f:1\nPress any key..",(NUMFRAMES*128*100*3)/totsize)


AVI_End()					-- Release the AVI output engine
free(frame)					-- Free memory

while get_key()=-1 do
end while

